/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef __ARTICLE_H__
#define __ARTICLE_H__

#define STATE_NONE 0
#define STATE_MULTIPART_ALL 1
#define STATE_MULTIPART_SOME 2
#define STATE_DOWNLOAD_QUEUED 4
#define STATE_DECODE_QUEUED 8
#define STATE_READ 16
#define STATE_DECODED 32
#define STATE_ERROR 64
#define STATE_NEW 128
#define STATE_CACHED 256
#define STATE_DECODE_FAILED 512

typedef struct _article_data article_data;
struct _article_data {
/* basic info */
	gchar *number;
	gchar *subject;
	gchar *author;
	gchar *message_id;
	gchar *references; /* ??? */
	time_t date;
	gint linecount;

/* multipart info */
	gint parts; /* no. of parts to whole multipart message */
	gint part;  /* current part no. */

/* threading */
	gint nthreads;   /* no. of threads underneath */
	GSList *threads; /* linked list of threads */

/* group */
	//group_data *group;
	gint state;
	//GtkCTreeNode *node;
};

gchar* extract_header		(article_data *, gchar *);
void article_free		(article_data *);
void article_free_protected	(article_data *);
void article_update		(article_data*, gboolean sync);
article_data* article_copy	(article_data *);
void article_delete		(void);
void article_add_flag		(article_data *, int, gboolean sync);
void article_remove_flag	(article_data *, int, gboolean sync);

int  article_find		(article_data *adata, gchar *message_id);
void article_delete_body	(article_data *adata);
article_data* article_get_current (void);

#endif /* __ARTICLE_H__ */
