/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <sys/types.h>
#include <sys/wait.h>
#include "pan.h"

extern int errno;

static
void
decode_open (char *fname)
{
	char *mime_type;
	char *mime_prog;
	char *tmp;

	mime_type = (char *)gnome_mime_type (fname);
	if (mime_type) {
		mime_prog = (char *)gnome_mime_program (mime_type);
		if (mime_prog) {
			mime_prog = g_strdup (mime_prog);
			tmp = strstr (mime_prog, "%f");
			if (tmp) {
				tmp[1] = 's';
				tmp = g_malloc (strlen (fname) + strlen (mime_prog));
				sprintf (tmp, mime_prog, fname);

				gnome_execute_shell(NULL, tmp);
				g_free (tmp);
			}
			g_free (mime_prog);
		}
	}
}

int
article_decode (article_data *adata, GSList *list, gboolean open)
{
	char path[PATH_MAX];
	gchar *ddir;

	uulist *item;
	GSList *p;
	static int count = 0;
	int rc = -1;

	/* setup for decoding */
	if ((ddir = group_get_attrib_string (Pan.active_group, "download_dir")) != NULL)
		chdir (ddir);
	else
		chdir (download_dir);
	g_free (ddir);

	UUInitialize ();
	UUSetMsgCallback (adata, UUD_message_callback);
	UUSetBusyCallback (adata, UUD_busy_callback, 1);
	UUSetFNameFilter (adata, UUD_file_name_filter);

	for (p = list; p; p = p->next) {
		g_snprintf (path, PATH_MAX, "/%s/k%s.body", data_dir, (char *)p->data);
		if ((UULoadFile (path, NULL, 0)) != UURET_OK) {
			pan_error_dialog (_("Decoded couldn't load temporary file:\n%s"), path);
			UUCleanUp();
			for (p = list; p; p = p->next) {
				acache_delete (p->data);
				g_free (p->data);
			}
			g_slist_free (list);
			return -1;
		}
		count++;
	}

	/* decode */
	if ((item = UUGetFileListItem(0)) != NULL) {
		if ((item->state & UUFILE_OK) != 0) {
			gui_set_status_va (_("Decoding article '%s'"), adata->subject);
			if ((UUDecodeFile (item, NULL)) != UURET_OK) {
//				gui_set_info_va (_("Error decoding '%s'"), adata->subject);
				log_add_va (_("Error decoding '%s'"), adata->subject);
				rc = -1;
			}
			else {
//				gui_set_info_va (_("Successfully decoded '%s'"), UUFNameFilter(item->filename));
				log_add_va ("Successfully decoded '%s'", UUFNameFilter(item->filename));
				rc = 0;
				if (open == TRUE) {
					decode_open(item->filename);
				}
			}
		}
		else {
//			gui_set_info_va (_("Error decoding '%s'"), adata->subject);
			log_add_va (_("Error decoding message '%s'"), adata->subject);
			rc = -1;
		}
	}

	/* clean up allocated variables */
	UUCleanUp ();
	for (p = list; p; p = p->next) {
		acache_delete (p->data);
		g_free (p->data);
	}
	g_slist_free (list);

	return rc;
}


/***************************************************************
* UUdeview Lib Stuff 
***************************************************************/
void
UUD_message_callback (void *opaque, char *msg, int level)
{
	if ((level == UUMSG_WARNING) || (level == UUMSG_ERROR)) {
		pan_error_dialog (_("An error occured during decoding.\n%s"), msg);
	}
}


int
UUD_busy_callback (void *opaque, uuprogress *progress)
{
	gfloat percent;

	percent = (progress->percent + 100.0 * (progress->partno-1)) / (progress->numparts * 100.0);

	gui_set_progress (percent);

	if (stop_all)
		return -1;
	else
		return 0;
}


char *
UUD_file_name_filter (void *opaque, char *fname)
{
	char *ptr;

	if (fname == NULL)
		return NULL;

	/*
	 * strip directory information
	 */

	if ((ptr = _FP_strrchr(fname, '/')) != NULL)
		return ptr + 1;
	else if ((ptr = _FP_strrchr(fname, '\\')) != NULL)
		return ptr + 1;

	return fname;
}
