/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/* Druid first run replacement
 * Jason Leach
 */

#include <gdk_imlib.h>
#include "pan.h"
#include "xpm/pan_druid.xpm"


/*---[ druid_finish ]-------------------------------------------------
 * 'finish' button has been clicked, grab and store all the data
 * that the user entered in.
 *--------------------------------------------------------------------*/
void
druid_finish (GnomeDruidPage *page, GnomeDruid *druid, gpointer window)
{
	gchar *profile_name;
	gchar *server_address;
	gchar *server_port;
	gint auth_toggle;
	gchar *server_username;
	gchar *server_password;
	gchar *full_name;
	gchar *email;
	gchar *organization;
	druid_data *ddata;

	ddata = gtk_object_get_data (GTK_OBJECT (window), "data");

	profile_name = gtk_entry_get_text (GTK_ENTRY(ddata->profile_name));
	server_address = gtk_entry_get_text (GTK_ENTRY(ddata->server_address));
	server_port = gtk_entry_get_text (GTK_ENTRY(ddata->server_port));
	auth_toggle = GTK_TOGGLE_BUTTON (ddata->auth_toggle)->active;
	server_username = gtk_entry_get_text (GTK_ENTRY(ddata->server_username));
	server_password = gtk_entry_get_text (GTK_ENTRY(ddata->server_password));
	full_name = gtk_entry_get_text (GTK_ENTRY(ddata->full_name));
	email = gtk_entry_get_text (GTK_ENTRY(ddata->email));
	organization = gtk_entry_get_text (GTK_ENTRY(ddata->organization));

	if (profile_name && server_address && server_port)
	{
		gchar* buf;

		gnome_config_set_string("/Pan/Servers/Names", profile_name );

		gnome_config_set_string("/Pan/Servers/Active", profile_name );

		buf = g_strdup_printf ( "/Pan/%s/Address", profile_name );
		gnome_config_set_string(buf, server_address);
		g_free ( buf );

		buf = g_strdup_printf ( "/Pan/%s/Port", profile_name);
		gnome_config_set_string(buf, server_port);
		g_free ( buf );

		buf = g_strdup_printf ( "/Pan/%s/Auth_Required", profile_name );
		gnome_config_set_int(buf, auth_toggle);
		g_free ( buf );

		if (auth_toggle) //server_username && server_password)
		{
			buf = g_strdup_printf ( "/Pan/%s/Username", profile_name );
			gnome_config_set_string(buf, server_username);
			g_free ( buf );

			buf = g_strdup_printf ( "/Pan/%s/Password", profile_name );
			gnome_config_set_string(buf, server_password);
			g_free ( buf );
		}
	}

	gnome_config_set_string ("/Pan/User/Full_Name", full_name);
	gnome_config_set_string ("/Pan/User/Email", email);
	gnome_config_set_string ("/Pan/User/Organization", organization);
	gnome_config_sync();

	gtk_widget_destroy (GTK_WIDGET (window));

	pan_init ();
}


/*---[ druid_next ]---------------------------------------------------
 * 'next' was clicked, flip to next wizard page, and set the previous
 * page while we're at it.
 *--------------------------------------------------------------------*/
void
druid_next (GnomeDruidPage *page, GnomeDruid *druid, GnomeDruidPage *next)
{
	gtk_object_set_data (GTK_OBJECT (next), "back", page);
	gnome_druid_set_page (druid, next);
}

static void
druid_auth_toggle_set (GtkWidget *widget, druid_data *ddata)
{
        if (!GTK_TOGGLE_BUTTON (widget)->active) {
		gtk_widget_set_sensitive (ddata->server_username_label, 0);
                gtk_widget_set_sensitive (ddata->server_username, 0);
		gtk_widget_set_sensitive (ddata->server_password_label, 0);
                gtk_widget_set_sensitive (ddata->server_password, 0);
        }
        else
        {
                /* disable everything */
		gtk_widget_set_sensitive (ddata->server_username_label, 1);	
                gtk_widget_set_sensitive (ddata->server_username, 1);
		gtk_widget_set_sensitive (ddata->server_password_label, 1);
                gtk_widget_set_sensitive (ddata->server_password, 1);
        }

/* JEL: this really necessary? A user reported 'clicking on need password in the druid'
   as hanging Pan, requiring him to kill it from console... somehow this seems like it
   could be related */
//        while (gtk_events_pending ()) /* do redrawing if necessary */
//                gtk_main_iteration ();
}


#if 0
GtkWidget *
druid_server_page2 (druid_data *ddata)
{
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *frame;
	GtkWidget *vbox;
	GtkWidget *vbox2;

	vbox = gtk_vbox_new (FALSE, GNOME_PAD);

	table = gtk_table_new (3, 2, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD_SMALL);
        gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
        gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	frame = gtk_frame_new (_("Server Information"));
	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD);
	gtk_container_add (GTK_CONTAINER (frame), table);
	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);


  /* Profile Name */
	label = gtk_label_new (_("Profile Name"));
	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);

	ddata->profile_name = gtk_entry_new_with_max_length(32);
	gtk_table_attach (GTK_TABLE (table), ddata->profile_name, 1, 2, 0, 1, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);

  /* Server Address */
	label = gtk_label_new (_("Server Address"));
	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);

	ddata->server_address = gtk_entry_new_with_max_length (32);
	gtk_table_attach (GTK_TABLE (table), ddata->server_address, 1, 2, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);

  /* Server Port */
	label = gtk_label_new (_("Server Port"));
	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);

	ddata->server_port = gtk_entry_new_with_max_length (32);
	gtk_entry_set_text (GTK_ENTRY(ddata->server_port), "119");
	gtk_table_attach (GTK_TABLE (table), ddata->server_port, 1, 2, 2, 3, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);


/* Authorization frame */
	frame = gtk_frame_new (_("Authorization"));
	vbox2 = gtk_vbox_new (FALSE, 0);
	table = gtk_table_new (2, 2, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD_SMALL);
        gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
        gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	gtk_container_add (GTK_CONTAINER (frame), vbox2);
	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);

  /* needs autorization information checkbox */
	ddata->auth_toggle = gtk_check_button_new_with_label (_("My server requires my username and password"));
	gtk_signal_connect (GTK_OBJECT (ddata->auth_toggle), "toggled",
			    GTK_SIGNAL_FUNC (druid_auth_toggle_set),
			    ddata);
	gtk_box_pack_start (GTK_BOX (vbox2), ddata->auth_toggle, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox2), table, FALSE, FALSE, 0);

  /* Server Username */
	ddata->server_username_label = gtk_label_new (_("Username"));
	gtk_misc_set_alignment (GTK_MISC (ddata->server_username_label), 1.0, 0.5);
	gtk_widget_set_sensitive (ddata->server_username_label, 0);
	gtk_table_attach (GTK_TABLE (table), ddata->server_username_label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);

	ddata->server_username = gtk_entry_new_with_max_length (32);
	gtk_widget_set_sensitive (ddata->server_username, 0);
	gtk_table_attach(GTK_TABLE (table), ddata->server_username, 1, 2, 0, 1, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);

  /* Server Password */
	ddata->server_password_label = gtk_label_new (_("Password"));
	gtk_misc_set_alignment (GTK_MISC (ddata->server_password_label), 1.0, 0.5);
	gtk_widget_set_sensitive (ddata->server_password_label, 0);
	gtk_table_attach (GTK_TABLE (table), ddata->server_password_label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);

	ddata->server_password = gtk_entry_new_with_max_length (32);
	gtk_entry_set_visibility (GTK_ENTRY(ddata->server_password), FALSE);
	gtk_widget_set_sensitive (ddata->server_password, 0);
	gtk_table_attach (GTK_TABLE (table), ddata->server_password, 1, 2, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);

	return vbox;
}
#endif


/*---[ druid_server_page ]--------------------------------------------
 * build and return the wizard page that gathers the server information
 *--------------------------------------------------------------------*/
GtkWidget*
druid_server_page (druid_data *ddata)
{
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *vbox;
	gchar *env_nntp_adr;

	vbox = gtk_vbox_new (FALSE, 0);

	table = gtk_table_new (7, 2, FALSE);
	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);

	gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);

	label = gtk_label_new (_("Let's begin by making a profile for your news server.\n\n"));

	gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 2, 0, 1,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

/* Server Profile Name */
	label = gtk_label_new (_("Server Profile Name:"));
	gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 1, 2,
			  GTK_FILL, GTK_FILL, 4, 0);

	ddata->profile_name = gtk_entry_new();
	gtk_entry_set_text (GTK_ENTRY(ddata->profile_name), _("Default"));
	gtk_table_attach (GTK_TABLE(table), ddata->profile_name, 1, 2, 1, 2,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

/* Server Address */
	label = gtk_label_new (_("Server Address:"));
	gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 2, 3,
			  GTK_FILL, GTK_FILL, 4, 0);

	ddata->server_address = gtk_entry_new();
	if ((env_nntp_adr = g_getenv ("NNTPSERVER")) != NULL)
		gtk_entry_set_text (GTK_ENTRY(ddata->server_address), env_nntp_adr);
	else
		gtk_entry_set_text (GTK_ENTRY(ddata->server_address), "news");
	gtk_table_attach (GTK_TABLE(table), ddata->server_address, 1, 2, 2, 3,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);
	
/* Server Port */
	label = gtk_label_new (_("Server Port:"));
	gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 3, 4,
			  GTK_FILL, GTK_FILL, 4, 0);

	ddata->server_port = gtk_entry_new();
	gtk_entry_set_text (GTK_ENTRY(ddata->server_port), "119");
	gtk_table_attach (GTK_TABLE(table), ddata->server_port, 1, 2, 3, 4,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

/* authorization required checkbox */
	ddata->auth_toggle = gtk_check_button_new_with_label (_("My server requires my username and password"));
	gtk_signal_connect (GTK_OBJECT (ddata->auth_toggle), "toggled",
			    GTK_SIGNAL_FUNC (druid_auth_toggle_set),
			    ddata);
	gtk_table_attach (GTK_TABLE(table), ddata->auth_toggle, 0, 2, 4, 5,
			  GTK_FILL, GTK_FILL, 4, 0);

/* Server Username */
	ddata->server_username_label = gtk_label_new (_("Username:"));
	gtk_misc_set_alignment(GTK_MISC(ddata->server_username_label), 0.0, 0.5);
	gtk_widget_set_sensitive (ddata->server_username_label, 0);
	gtk_table_attach(GTK_TABLE(table), ddata->server_username_label, 0, 1, 5, 6,
			 GTK_FILL, GTK_FILL, 4, 0);

	ddata->server_username = gtk_entry_new();
//	gtk_entry_set_text (GTK_ENTRY(ddata->server_username), "");
	gtk_widget_set_sensitive (ddata->server_username, 0);
	gtk_table_attach (GTK_TABLE(table), ddata->server_username, 1, 2, 5, 6,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

/* Server Password */
	ddata->server_password_label = gtk_label_new (_("Password:"));
	gtk_misc_set_alignment(GTK_MISC(ddata->server_password_label), 0.0, 0.5);
	gtk_widget_set_sensitive (ddata->server_password_label, 0);
	gtk_table_attach(GTK_TABLE(table), ddata->server_password_label, 0, 1, 6, 7,
			 GTK_FILL, GTK_FILL, 4, 0);

	ddata->server_password = gtk_entry_new();
//	gtk_entry_set_text (GTK_ENTRY(ddata->server_password), "");
	gtk_widget_set_sensitive (ddata->server_password, 0);
	gtk_entry_set_visibility (GTK_ENTRY(ddata->server_password), FALSE);
	gtk_table_attach (GTK_TABLE(table), ddata->server_password, 1, 2, 6, 7,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	return vbox;
}


/*---[ druid_spawn ]--------------------------------------------------
 * spawn the first-run wizard
 *--------------------------------------------------------------------*/
void
druid_spawn (void)
{
	GtkWidget *window;
	GtkWidget *druid;
	GtkWidget *page_start;
	GtkWidget *page_a;
	GtkWidget *page_b;
	GtkWidget *page_finish;
	GdkImlibImage *logo = NULL;
	GdkImlibImage *watermark = NULL;
	gchar *fname;
	druid_data *ddata;
		
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER_ALWAYS);

	druid = gnome_druid_new ();
	ddata = g_new (druid_data, 1);

	fname = gnome_pixmap_file ("gnome-logo-icon.png");
	if (fname)
		logo = gdk_imlib_load_image (fname);
	g_free (fname);

	watermark = gdk_imlib_create_image_from_xpm_data (pan_druid_xpm);

	gtk_object_set_data (GTK_OBJECT(window), "data", ddata);

	/* The druid pages */
	page_start = gnome_druid_page_start_new_with_vals (
		_("Pan Setup"),
		_("Welcome to Pan!\nSince this is your first time running Pan,\nlet's setup a few things.\n"),
		logo,
		watermark);
	page_a = gnome_druid_page_standard_new_with_vals (_("NNTP Server Information"), logo);
	page_b = gnome_druid_page_standard_new_with_vals (_("User Information"), logo);
	page_finish = gnome_druid_page_finish_new_with_vals (
		_("Setup Complete"),
		_("Pan is now configured and ready to be run.\nPlease send any comments, bug reports,\nor suggestions to pan@superpimp.org."),
		logo,
		watermark);
	gtk_box_pack_start (GTK_BOX (GNOME_DRUID_PAGE_STANDARD (page_a)->vbox),
			    druid_server_page(ddata), FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (GNOME_DRUID_PAGE_STANDARD (page_b)->vbox),
			    druid_user_page(ddata), FALSE, FALSE, 0);

	gtk_signal_connect (GTK_OBJECT (page_a), "next",
			    GTK_SIGNAL_FUNC (druid_next),
			    (gpointer) page_b);
	gtk_signal_connect (GTK_OBJECT (page_b), "next",
			    GTK_SIGNAL_FUNC (druid_next),
			    (gpointer) page_finish);
	gtk_signal_connect (GTK_OBJECT (page_finish), "finish",
			    GTK_SIGNAL_FUNC (druid_finish),
			    window);

	gtk_signal_connect_object (GTK_OBJECT (druid), "cancel",
				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
				   GTK_OBJECT (window));

	/* tie it all together */
	gtk_container_add (GTK_CONTAINER (window), druid);		
	gnome_druid_append_page (GNOME_DRUID (druid),
				 GNOME_DRUID_PAGE (page_start));
	gnome_druid_append_page (GNOME_DRUID (druid),
				 GNOME_DRUID_PAGE (page_a));
	gnome_druid_append_page (GNOME_DRUID (druid),
				 GNOME_DRUID_PAGE (page_b));
	gnome_druid_append_page (GNOME_DRUID (druid),
				 GNOME_DRUID_PAGE (page_finish));

	/* set the first page */
	gnome_druid_set_page (GNOME_DRUID (druid),
			      GNOME_DRUID_PAGE (page_start));

	gtk_widget_show_all (window);
}


/*---[ druid_user_page ]----------------------------------------------
 * build and return the wizard page that gathers the user information
 *--------------------------------------------------------------------*/
GtkWidget *
druid_user_page (druid_data *ddata)
{
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *vbox;

	vbox = gtk_vbox_new (FALSE, 0);

	table = gtk_table_new (4, 2, FALSE);
	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
	gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);

	label = gtk_label_new (_("The following user information would be nice to have.\n\n"));
	gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 2, 0, 1,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	label = gtk_label_new(_("Full Name:"));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
			 GTK_FILL, GTK_FILL, 4, 0);

	ddata->full_name = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE(table), ddata->full_name, 1, 2, 1, 2,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	label = gtk_label_new (_("Email Address:"));
	gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 2, 3,
			  GTK_FILL, GTK_FILL, 4, 0);

	ddata->email = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE(table), ddata->email, 1, 2, 2, 3,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	label = gtk_label_new (_("Organization:"));
	gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 3, 4,
			  GTK_FILL, GTK_FILL, 4, 0);

	ddata->organization = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE(table), ddata->organization, 1, 2, 3, 4,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	return vbox;
}
