/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <errno.h>
#include "xpm/disk.xpm"
#include "pan.h"

GnomePixmap *disk_pixmap = NULL;

static char *data_base = NULL;
static group_data *memblock = NULL;
static GnomeUIInfo groups_menu_popup[] = {
	{ GNOME_APP_UI_ITEM, N_("Subscribe to Selected"), N_("Subscribe to the selected group(s)"), grouplist_selected_subscribe },
	{ GNOME_APP_UI_ITEM, N_("Unsubscribe from Selected"), N_("Unsubscribe from the selected group(s)"), grouplist_selected_unsubscribe },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Get All Article Headers"), N_("Get all article headers"), grouplist_selected_download_all },
	{ GNOME_APP_UI_ITEM, N_("Get New Article Headers"), N_("Get new article headers"), grouplist_selected_download_new },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Empty Group"), N_("Remove all articles from the selected group(s)."), grouplist_selected_empty },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Remove `New' Status"), N_("Remove selected group(s) from `new groups' list."), grouplist_selected_unnew },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Properties"), N_("Set the properties for the selected group."), grouplist_selected_properties },
	GNOMEUIINFO_END
};


void
grouplist_show_subscribed (void)
{
	group_mode = GROUP_SUBSCRIBED;
	grouplist_update ();
}

void
grouplist_show_all (void)
{
	group_mode = GROUP_ALL;
	grouplist_update ();
}

void
grouplist_show_new (void)
{
	group_mode = GROUP_NEW;
	grouplist_update ();
}


/*---[ select_group_row ]---------------------------------------------
 * process what happens when a user double clicks on a group in the
 * group list
 *--------------------------------------------------------------------*/
void
grouplist_select_row (GtkWidget *widget, int r, int c,
		      GdkEvent *event)
{
	group_data *gdata;
	GdkEventButton *bevent = (GdkEventButton *)event;

	if (bevent)
	{
		if ((bevent->button) == 1 && (bevent->type == GDK_2BUTTON_PRESS))
		{
			gdata = (group_data *) gtk_clist_get_row_data(GTK_CLIST(widget), r);
			
			if (Pan.active_group == gdata) {
				gui_page_set (ARTICLELIST_PAGE, Pan.article_ctree);
				return;
			}

			/* if in notebook layout, flip to the article list */
			gui_page_set (ARTICLELIST_PAGE, Pan.article_ctree);
			group_set_active_group ( gdata );//gui_set_title (gdata);
		}
	}
}


void
grouplist_process_selected (int type)
{
	GList *l;
	group_data *gdata;

	l = GTK_CLIST (Pan.group_clist)->selection;

	if ( (!l) && (Pan.active_group) ) {
		switch (type) {
		case 3:
			queue_add_articlelist_download (Pan.active_group,
							QUEUE_ARTICLELIST_DOWNLOAD_ALL | QUEUE_BACKGROUND);
			break;
		case 4:
			queue_add_articlelist_download (Pan.active_group,
							QUEUE_ARTICLELIST_DOWNLOAD_NEW | QUEUE_BACKGROUND);
			break;
		default:
			break;
		}
	}

	for (l = GTK_CLIST (Pan.group_clist)->selection; l; l = l->next) {
		gdata = (group_data *) gtk_clist_get_row_data (GTK_CLIST (Pan.group_clist),
							       GPOINTER_TO_INT (l->data));
	
		if (gdata) {
			switch (type) {
			case 1: /* subscribe */
				gdata->flags |= GROUP_SUBSCRIBED;
				break;
			case 2: /* unsubscribe */
				gdata->flags &= ~GROUP_SUBSCRIBED;
				group_empty_dialog (gdata);
				break;
			case 3: /* download all */
				queue_add_articlelist_download (gdata,
								QUEUE_ARTICLELIST_DOWNLOAD_ALL |
								QUEUE_BACKGROUND);
				break;
			case 4: /* download new */
				queue_add_articlelist_download (gdata,
								QUEUE_ARTICLELIST_DOWNLOAD_NEW |
								QUEUE_BACKGROUND);
				break;
			case 5: /* clean/empty */
				group_empty_dialog (gdata);
				break;
			case 6: /* group properties */
				group_props_spawn (gdata);
				clist_unselect_all (Pan.group_clist);
				return;
				break;
			case 7: /* remove "new" mark */
				gdata->flags &= ~GROUP_NEW;
				break;
			default:
				break;
			}
		}
	}

	clist_unselect_all (Pan.group_clist);

//	if (((type == 2) || (type == 5)) && (group_mode == GROUP_SUBSCRIBED))
//		grouplist_update ();
	if (type == 7 && (group_mode == GROUP_NEW))
		grouplist_update ();

	if ((type == 1) || (type == 2) || (type == 7))
		grouplist_save_index (Pan.grouplist);
}

void grouplist_selected_subscribe (void) { grouplist_process_selected (1); }
void grouplist_selected_unsubscribe (void) { grouplist_process_selected (2); }
void grouplist_selected_download_all (void) { grouplist_process_selected (3); }
void grouplist_selected_download_new (void) { grouplist_process_selected (4); }
void grouplist_selected_empty (void) { grouplist_process_selected (5); }
void grouplist_selected_properties (void) { grouplist_process_selected (6); }
void grouplist_selected_unnew (void) { grouplist_process_selected (7); }


void
grouplist_get_all (void)
{
	GList *l;
	group_data *gdata;

	l = GTK_CLIST (Pan.group_clist)->selection; 
	
	if (l == NULL) {
		if (Pan.active_group)
			queue_add_articlelist_download (Pan.active_group,
							QUEUE_ARTICLELIST_DOWNLOAD_ALL | QUEUE_FOREGROUND);
	}
	else if (l->next == NULL) {
			gdata = (group_data *) gtk_clist_get_row_data (GTK_CLIST (Pan.group_clist), GPOINTER_TO_INT (l->data));
			queue_add_articlelist_download (gdata,
							QUEUE_ARTICLELIST_DOWNLOAD_ALL | QUEUE_FOREGROUND);
	}
	else {
		for (; l; l = l->next) {
			gdata = (group_data *) gtk_clist_get_row_data (GTK_CLIST (Pan.group_clist), GPOINTER_TO_INT (l->data));
			queue_add_articlelist_download (gdata,
							QUEUE_ARTICLELIST_DOWNLOAD_ALL | QUEUE_BACKGROUND);
		}
	}
	clist_unselect_all (Pan.group_clist);
}


void
grouplist_get_new (void)
{
	GList *l;
	group_data *gdata;

	l = GTK_CLIST (Pan.group_clist)->selection; 
	
	if (l == NULL) {
		if (Pan.active_group)
			queue_add_articlelist_download (Pan.active_group, QUEUE_ARTICLELIST_DOWNLOAD_NEW | QUEUE_FOREGROUND);
	} else if (l->next == NULL) {
			gdata = (group_data *) gtk_clist_get_row_data (GTK_CLIST (Pan.group_clist), GPOINTER_TO_INT (l->data));
			queue_add_articlelist_download (gdata, QUEUE_ARTICLELIST_DOWNLOAD_NEW | QUEUE_FOREGROUND);
	} else {
		for (; l; l = l->next) {
			gdata = (group_data *) gtk_clist_get_row_data (GTK_CLIST (Pan.group_clist), GPOINTER_TO_INT (l->data));
			queue_add_articlelist_download (gdata, QUEUE_ARTICLELIST_DOWNLOAD_NEW | QUEUE_BACKGROUND);
		}
	}
	clist_unselect_all (Pan.group_clist);
}

void
grouplist_select_all (void)
{
	gtk_clist_select_all (GTK_CLIST (Pan.group_clist));
}


void
grouplist_button_press (GtkWidget *widget, GdkEventButton *bevent)
{
	if (bevent->button != 1)
		gtk_signal_emit_stop_by_name (GTK_OBJECT(widget), "button_press_event");
	
	if (bevent->button == 3)
	{
		const int sel = GTK_CLIST(Pan.group_clist)->selection != NULL;

		GList* l = GTK_MENU_SHELL(groups_clist_menu)->children; /* l is sub */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );
		l = l->next; /* l is unsub */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );
		l = l->next; /* l is separator */
		l = l->next; /* l is "get all headers" */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );
		l = l->next; /* l is "get new headers" */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );
		l = l->next; /* l is separator */
		l = l->next; /* l is "empty group" */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );
		l = l->next; /* l is separator */
		l = l->next; /* l is "unnew" */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );
		l = l->next; /* l is separator */
		l = l->next; /* l is "properties" */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );

		gtk_menu_popup (GTK_MENU(groups_clist_menu), NULL, NULL, NULL,
				NULL, bevent->button, bevent->time);
	}
}


/*---[ grouplist_create ]---------------------------------------------
 *
 *--------------------------------------------------------------------*/
GtkWidget *
grouplist_create (void)
{
	GtkWidget *swindow;

	char *titles[] = {
		_("All Groups"),
		_("Unread"),
		_("Total")
	};

	Pan.group_clist = gtk_clist_new_with_titles (3, titles);

	widget_set_font ( GTK_WIDGET(Pan.group_clist), grouplist_font );

	swindow = gtk_scrolled_window_new (NULL, NULL);

	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER(swindow), Pan.group_clist);

	gtk_signal_connect (GTK_OBJECT (Pan.group_clist),
			    "select_row",
			    GTK_SIGNAL_FUNC (grouplist_select_row), NULL);

	gtk_signal_connect (GTK_OBJECT (Pan.group_clist),
			    "unselect_row",
			    GTK_SIGNAL_FUNC (grouplist_select_row), NULL);

	gtk_signal_connect (GTK_OBJECT (Pan.group_clist),
			    "button_press_event",
			    GTK_SIGNAL_FUNC (grouplist_button_press), NULL);

	gtk_clist_set_column_width (GTK_CLIST (Pan.group_clist), 0, 200);
//	gtk_clist_set_column_auto_resize (GTK_CLIST (Pan.group_clist), 1, TRUE);
//	gtk_clist_set_column_auto_resize (GTK_CLIST (Pan.group_clist), 2, TRUE);

/* JEL: this works well in paned mode, but not so well in notebook mode, we
   need to find a solution that will cover both ... maybe splitting the justification
   parts into the _construct funcs for both ...
	gtk_clist_set_column_justification ( GTK_CLIST(Pan.group_clist), 0, GTK_JUSTIFY_LEFT );
	gtk_clist_set_column_justification ( GTK_CLIST(Pan.group_clist), 1, GTK_JUSTIFY_RIGHT );
	gtk_clist_set_column_justification ( GTK_CLIST(Pan.group_clist), 2, GTK_JUSTIFY_RIGHT );
*/
	gtk_clist_set_selection_mode (GTK_CLIST (Pan.group_clist),
				      GTK_SELECTION_EXTENDED);

	gtk_signal_connect (GTK_OBJECT (Pan.group_clist), "click_column",
			    GTK_SIGNAL_FUNC (click_groups_column_cb), NULL);

	group_mode = gnome_config_get_int ("/Pan/State/group_mode=0");

	/* Create the Popup Menu while we are here. */
	groups_clist_menu = gnome_popup_menu_new (groups_menu_popup);	

	/* Lets have the Pixmap ready too */
	disk_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d (disk_xpm);

	return swindow;
}


/*---[ grouplist_free ]-----------------------------------------------
 * Free the allocated memory of an entire grouplist.  Called when
 * switching servers refreshing the group list.
 *--------------------------------------------------------------------*/
void
grouplist_free (GSList *list)
{
	/* Warning:  Don't use this on the Pan.grouplist
	 * constructed by grouplist_load */
	if (list) {
		g_slist_foreach (list, (GFunc) group_free, NULL);
		g_slist_free (list);
		list = NULL;
	}
}


/*---[ grouplist_grep ]-----------------------------------------------
 * grep the group list for whatever the user typed into Find: when the
 * user hits enter in the Find: box
 *--------------------------------------------------------------------*/
void
grouplist_grep (GtkWidget *group_entry)
{
	gchar *entry_text;
	GSList *pos;
	group_data *gdata;

	g_return_if_fail (Pan.group_clist);

	gtk_clist_clear (GTK_CLIST (Pan.group_clist));
	entry_text = gtk_entry_get_text (GTK_ENTRY (group_entry));
	gtk_clist_freeze (GTK_CLIST (Pan.group_clist));
	pos = Pan.grouplist;
	while (pos) {
		gdata = pos->data;
		if (((gdata->flags & group_mode) ||
		     (group_mode == GROUP_ALL)) && strstr (gdata->name, entry_text))
			add_group_to_clist (pos->data);

		pos = pos->next;
	}
	gtk_clist_thaw (GTK_CLIST (Pan.group_clist));
}



/* index/data groups format notes:

   for each groups list 2 files are made:
     ~/.pan/data/xxx/index
     ~/.pan/data/xxx/data

   The index file consists of an array of int's. The first is the number
   of groups. From there on in the int's come in pairs, the first being
   the 'flags' and the second an offset into data of the name

   The data file consists of NUL terminated strings end on end.

   On loading the data file is read into a global buffer 'data_base'
   and an array of group_data's is created, one for each group in index
   (as set by the first int of index). Each int pair is then read from index
   and a group_data is filled out by setting the flag and adding data_base
   to offset the make a valid gchar *. data_base is g_free()'ed by
   grouplist_load if set, thus only one grouplist can be loaded at one time
   currently.

   --AGL
*/
void
grouplist_load (void)
{
	FILE *fp_index, *fp_data;
	gchar *path;
	int i, j, in[2], len;
	group_data *gdata;
	GnomeDialog *dialog;
	GtkWidget *label;
//	struct timeval tv;
	GSList *groups_list = NULL;
	
	g_return_if_fail (Pan.active_server && Pan.active_server->name);

//	gettimeofday (&tv, NULL);
//	printf ("Load starts at %d.%d\n", tv.tv_sec, tv.tv_usec);

	g_slist_free(Pan.grouplist);
	Pan.grouplist = NULL;
	g_free (data_base);
	data_base = NULL;
	g_free(memblock);
	memblock = NULL;
	
	path = g_strdup_printf ("/%s/%s/index", data_dir,
				Pan.active_server->name);

	if ((fp_index = fopen(path, "r")) == NULL) {
		g_free (path);
		dialog = (GnomeDialog *) gnome_dialog_new (_("No Group List"), GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);
		label = gtk_label_new (_("We don't have a group list for the active server.\nShall we get one?\n"));
		gtk_container_add (GTK_CONTAINER(dialog->vbox), label);
		gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
				    GTK_SIGNAL_FUNC(group_dialog_clicked_cb), NULL);

		gui_popup_draw (GTK_WIDGET(dialog));
		return;
	}
	g_free (path);

	/* I'd like to be able to g_malloc a big lump of data, but
	   gslist.c uses some strange alloc so I cant ;( --AGL */
	fread (&i, sizeof (i), 1, fp_index);

	path = g_strdup_printf ("/%s/%s/data", data_dir,
				Pan.active_server->name);
	
	fp_data = fopen (path, "r");
	if (fp_data == NULL) {
		pan_error_dialog (_("We couldn't open group list %s:\n%s"),
				  path, strerror(errno));
		g_free (path);
		return;
	}
	g_free (path);
	
	fseek (fp_data, 0, SEEK_END);
	len = ftell (fp_data);
	if (len <= 0) {
		pan_error_dialog (_("There was a problem loading the grouplist data"));
		return;
	}
	fseek (fp_data, 0, SEEK_SET);

	
	data_base = g_malloc (len);
	fread (data_base, len, 1, fp_data);

//	memblock = g_new0 (group_data, i);
	memblock = g_malloc0 (sizeof (group_data) * i);
	//memblock2 = g_new0 (GSList, i);
	for (j = 0; j < i; j++) {
		//cur = &memblock2[j];
		//cur->next = &memblock2[j + 1];
		gdata = &memblock[j];
		//cur->data = data;

		fread (&in, sizeof (int), 2, fp_index);
		gdata->flags = in[0];
		gdata->name = &data_base[in[1]];

		groups_list = g_slist_prepend (groups_list, gdata);
	}
	//memblock2[i - 1].next = 0;
	groups_list = g_slist_reverse (groups_list);
	//Pan.grouplist = memblock2;
	Pan.grouplist = groups_list;
	fclose (fp_data);
	fclose (fp_index);

//	gettimeofday (&tv, NULL);
//	printf ("Load ends at %d.%d\n", tv.tv_sec, tv.tv_usec);
}


void
grouplist_save_index (GSList *glist)
{
	FILE *fp;
	gchar *path;
	int i, j;
	GSList *cur;
	group_data *d;


	g_return_if_fail (glist);

	path = g_strdup_printf ("/%s/%s/index", data_dir,
				Pan.active_server->name);
	if ((fp = fopen(path, "w")) == NULL) {
		pan_error_dialog (_("We couldn't save group list to %s:\n%s"),
				  path, strerror(errno));
		g_free (path);
		return;
	}
	g_free (path);

	i = j = 0;
	fwrite (&i, sizeof (i), 1, fp);

	cur = glist;
	while (cur) {
	 	d = cur->data;
		fwrite (&d->flags, sizeof (d->flags), 1, fp);
		fwrite (&j, sizeof (j), 1, fp);
		i++;
		/* update offset */
		j += strlen (d->name) + 1; /* NULL at the end of string */

		cur = cur->next;
	}
	fseek (fp, 0, SEEK_SET);
	fwrite (&i, sizeof (int), 1, fp);
	fclose (fp);

}

/*---[ grouplist_save ]-----------------------------------------------
 * save the list of groups to the 'index' and 'data' files
 *--------------------------------------------------------------------*/
void
grouplist_save (GSList *glist)
{
	FILE *fp;
	gchar *path;
	int i, j;
	GSList *cur;
	group_data *d;

	g_return_if_fail (glist);

	path = g_strdup_printf ("/%s/%s/index", data_dir,
				Pan.active_server->name);
	if ((fp = fopen(path, "w")) == NULL) {
		pan_error_dialog (_("We couldn't save group list to %s:\n%s"),
				  path, strerror(errno));
		g_free (path);
		return;
	}
	g_free (path);

	i = j = 0;
	fwrite (&i, sizeof (i), 1, fp);

	cur = glist;
	while (cur) {
	 	d = cur->data;
		fwrite (&d->flags, sizeof (d->flags), 1, fp);
		fwrite (&j, sizeof (j), 1, fp);
		i++;
		/* update offset */
		j += strlen (d->name) + 1; /* NULL at the end of string */

		cur = cur->next;
	}
	fseek (fp, 0, SEEK_SET);
	fwrite (&i, sizeof (int), 1, fp);
	fclose (fp);

	path = g_strdup_printf ("/%s/%s/data", data_dir,
				Pan.active_server->name);
	if ((fp = fopen(path, "w")) == NULL) {
		pan_error_dialog (_("We couldn't save group list to %s:\n%s"),
				  path, strerror(errno));
		g_free (path);
		return;
	}
	g_free (path);

	cur = glist;
	while (cur) {
		d = cur->data;
		/* sizeof(char) == 1, right? --AGL */
		fwrite (d->name, 1, strlen (d->name) + 1, fp);
		cur = cur->next;
	}
	fclose (fp);
}


/*---[ grouplist_update ]---------------------------------------------
 * update the entire group CLIST, clearing and then filling it in
 * with the groups that are part of the current group mode (all, 
 * subscribed, etc.)
 *--------------------------------------------------------------------*/
void
grouplist_update (void)
{
	if (Pan.grouplist == NULL)
		grouplist_load();

	pan_lock ();
	gtk_clist_clear (GTK_CLIST(Pan.group_clist));
	switch (group_mode) {
	case GROUP_ALL:
		gtk_clist_set_column_title (GTK_CLIST(Pan.group_clist), 0, _("All Groups"));
		gtk_clist_freeze (GTK_CLIST (Pan.group_clist));
		g_slist_foreach (Pan.grouplist, (GFunc) add_group_to_clist, NULL);
		//add_info_line_to_clist ();
		gtk_clist_thaw (GTK_CLIST (Pan.group_clist));
		break;
	case GROUP_SUBSCRIBED:
		gtk_clist_set_column_title (GTK_CLIST(Pan.group_clist), 0, _("Subscribed Groups"));
		gtk_clist_freeze (GTK_CLIST (Pan.group_clist));
		g_slist_foreach (Pan.grouplist, (GFunc) add_subscribed_group_to_clist, NULL);
		gtk_clist_thaw (GTK_CLIST (Pan.group_clist));
		break;
	case GROUP_NEW:
		gtk_clist_set_column_title (GTK_CLIST(Pan.group_clist), 0, _("New Groups"));
		gtk_clist_freeze (GTK_CLIST (Pan.group_clist));
		g_slist_foreach (Pan.grouplist, (GFunc) add_new_group_to_clist, NULL);
		gtk_clist_thaw (GTK_CLIST (Pan.group_clist));
		break;
	default:
                g_assert_not_reached ( );
	}
	pan_unlock ();
}


#if 0
void
add_info_line_to_clist (void)
{
	char *text[4];
	int row;

	text[0] = "Enter a search string below";
	text[1] = text[2] = text[3] = NULL;

	row = gtk_clist_append (GTK_CLIST (Pan.group_clist), text);
}
#endif


/*---[ grouplist_update_row ]-----------------------------------------
 * called when a user double clicks on a group name, updates the clist
 * to reflect total messages and if the group is "on disk" (stored in
 * a database file)
 *--------------------------------------------------------------------*/
void
grouplist_update_row (const group_data * gdata)
{
	GtkCList* list = GTK_CLIST ( Pan.group_clist );
	gint row, total;

	pan_lock ();
	row = gtk_clist_find_row_from_data (GTK_CLIST(Pan.group_clist), (gpointer)gdata);
	gtk_clist_freeze (GTK_CLIST (Pan.group_clist));

	total = group_get_attrib_i ( gdata, "Total" );
	if ( total )
	{
		// total column
		gchar* p = commatize_ulong ( total );
		gtk_clist_set_text ( list, row, 2, p );
		g_free ( p );

		// unread column
		p = commatize_ulong ( total - group_get_attrib_i(gdata,"Read") );
		gtk_clist_set_text ( list, row, 1, p );
		g_free ( p );

		// group name
		gtk_clist_set_pixtext ( list, row, 0, gdata->name, 5, disk_pixmap->pixmap, disk_pixmap->mask);
	}
	else
	{
		gtk_clist_set_text ( list, row, 0, gdata->name);
		gtk_clist_set_text ( list, row, 1, "" );
		gtk_clist_set_text ( list, row, 2, "" );
	}


	gtk_clist_thaw (GTK_CLIST (Pan.group_clist));

	gtk_widget_queue_draw (GTK_WIDGET(Pan.group_clist));
	pan_unlock ();
}
