/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "pan.h"

#define layout_paned_text "_Paned Layout"
#define layout_paned_hint "Switch to the paned layout."
#define layout_notebook_text "_Notebook Layout"
#define layout_notebook_hint "Switch to the notebook layout."

static void layout_cb ( GtkButton* );

static int window_delete_event_cb ( GtkWidget*, GdkEvent*, gpointer );
static void exit_cb ( void );

/* 'File' menu */
static GnomeUIInfo file_menu[] = {
	GNOMEUIINFO_MENU_PRINT_ITEM (not_implemented, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM(exit_cb, NULL),
	GNOMEUIINFO_END
};

/* 'Edit' menu */
static GnomeUIInfo edit_menu[] = {
	GNOMEUIINFO_MENU_CUT_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_COPY_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_PASTE_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_SELECT_ALL_ITEM (gui_select_all, NULL),
	GNOMEUIINFO_MENU_CLEAR_ITEM (not_implemented, NULL),
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("_Find"), N_("Find text in displayed header fields"), articlelist_find_text_cb,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_SEARCH, 'F', GDK_CONTROL_MASK, NULL},
	{ GNOME_APP_UI_ITEM, N_("Find _Next"), N_("Repeat the last search."), articlelist_find_next_cb,
	  NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'G', GDK_CONTROL_MASK, NULL},
	GNOMEUIINFO_END
};

/* 'Settings' menu */
static GnomeUIInfo settings_menu[] = {
	GNOMEUIINFO_MENU_PREFERENCES_ITEM(prefs_spawn, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo servers_menu[] = {
	GNOMEUIINFO_END
};

/* 'Online' menu */
static GnomeUIInfo online_menu[] = {
	{ GNOME_APP_UI_ITEM, N_("Go _Online"), N_("Go into online mode."), nntp_want_connect, 0 },
	{ GNOME_APP_UI_ITEM, N_("Go O_ffline"), N_("Go into offline mode."), nntp_want_disconnect },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("_Stop all Online Tasks"), N_("Stop all online tasks."), stop_all_mf,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_STOP, 'S', GDK_CONTROL_MASK, NULL},
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Get _List of Groups"), N_("Refresh the listing of groups."), get_all_groups,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_REFRESH, 0, 0, NULL},
	GNOMEUIINFO_END
};

/* 'Help' menu */
GnomeUIInfo help_menu[] = {
	GNOMEUIINFO_ITEM_STOCK (N_("_About..."), N_("Display program information, version number, and copyright."),
				gui_about, GNOME_STOCK_MENU_ABOUT),
	GNOMEUIINFO_END
};


/* 'Windows' menu */
static GnomeUIInfo view_menu[] = {
	{ GNOME_APP_UI_ITEM, N_(layout_paned_text), N_(layout_paned_hint), (gpointer)layout_cb, (gpointer)GUI_PANED },
	{ GNOME_APP_UI_ITEM, N_(layout_notebook_text), N_(layout_notebook_hint), (gpointer)layout_cb, (gpointer)GUI_NOTEBOOK },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("_Log Viewer"), N_("Open up the Log Viewer."), log_viewer_spawn,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PREFERENCES, 'L', GDK_CONTROL_MASK, NULL},
//	{ GNOME_APP_UI_ITEM, N_("Queue Manager"), N_("Open up the Queue Manager."), queue_manager_spawn },
//	{ GNOME_APP_UI_ITEM, N_("_Statistics"), N_("View Pan's online statistics."), stats_window_spawn },
	GNOMEUIINFO_END
};

static GnomeUIInfo groupview_menu[] = {
	{ GNOME_APP_UI_ITEM, N_("All Groups"), N_("Show all available groups in the group list."), grouplist_show_all },
	{ GNOME_APP_UI_ITEM, N_("Subscribed Groups"), N_("Show only the subscribed groups in the group list."), grouplist_show_subscribed },
	{ GNOME_APP_UI_ITEM, N_("New Groups"), N_("Show only the new groups in the group list."), grouplist_show_new },
	GNOMEUIINFO_END
};

/* 'Group' menu */
static GnomeUIInfo group_menu[] = {
	{ GNOME_APP_UI_ITEM, N_("Get _All Article Headers"), N_("Get all article headers for selected groups."), grouplist_selected_download_all },
	{ GNOME_APP_UI_ITEM, N_("Get _New Article Headers"), N_("Get new article headers for selected groups."), grouplist_selected_download_new },
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_SUBTREE (N_("_Show Groups"), &groupview_menu),
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("_Properties"), N_("Set the properties for the selected group."), grouplist_selected_properties },
	GNOMEUIINFO_END
};

/* 'Message' menu */
static GnomeUIInfo message_menu[] = {
	{ GNOME_APP_UI_ITEM, N_("_New Message"), N_("Create a new message."), post_window_spawn,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL, 'N', GDK_CONTROL_MASK, NULL },
	GNOMEUIINFO_SEPARATOR,
#if 0
	{ GNOME_APP_UI_ITEM, N_("Message _Filter"), N_("Thread Tracking and Killfile Management."),
	  message_filter_ui, NULL,
	  NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PREFERENCES, 0, GDK_CONTROL_MASK, NULL },
#endif
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Reply to _Sender"), N_("Create a reply to the sender."), followup_window },
//	{ GNOME_APP_UI_ITEM, N_("Reply to All"), N_("Post a response to everyone"), followup_window },
	{ GNOME_APP_UI_ITEM, N_("Reply to _Group"), N_("Post a reply to the message on the news server."), followup_window },
	{ GNOME_APP_UI_ITEM, N_("_Forward"), N_("Forward this message to someone."), not_implemented },
	{ GNOME_APP_UI_ITEM, N_("F_orward as Attachment"), N_("Forward this message as an attachment to someone."), not_implemented },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("_Save Attachment"), N_("Download and decode attachments in the selected messages."), articlelist_selected_decode, NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'D', GDK_CONTROL_MASK, NULL},
	{ GNOME_APP_UI_ITEM, N_("_Open Attachment"), N_("Download, decode, and open attachments in the selected messages."), articlelist_selected_open, NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'O', GDK_CONTROL_MASK, NULL},
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("D_elete Message"), N_("Deletes the selected message from the database."), articlelist_selected_delete },
	{ GNOME_APP_UI_ITEM, N_("Mark All _Read"), N_("Mark all articles read"), articlelist_all_mark_read },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Select _All"), N_("Select all messages"), articlelist_select_all, NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'A', GDK_CONTROL_MASK, NULL},
	GNOMEUIINFO_END
};

static GnomeUIInfo pan_main_menu[] = {
	GNOMEUIINFO_MENU_FILE_TREE (file_menu),
	GNOMEUIINFO_MENU_EDIT_TREE (edit_menu),
	GNOMEUIINFO_MENU_SETTINGS_TREE (settings_menu),
	GNOMEUIINFO_SUBTREE (N_("_View"), &view_menu),
	GNOMEUIINFO_SUBTREE (N_("Se_rvers"), &servers_menu),
	GNOMEUIINFO_SUBTREE (N_("_Online"), &online_menu),
	GNOMEUIINFO_SUBTREE (N_("_Group"), &group_menu),
	GNOMEUIINFO_SUBTREE (N_("_Message"), &message_menu),
	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
	GNOMEUIINFO_END
};

/*
static GnomeUIInfo pan_main_toolbar[] = {
	{ GNOME_APP_UI_ITEM, N_("New Message"), N_("Create a new message."), post_window_spawn,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL, 'N', GDK_CONTROL_MASK, NULL},
	{ GNOME_APP_UI_ITEM, N_("Stop All"), N_("Stop all online tasks."), stop_all_mf,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_STOP, 'S', GDK_CONTROL_MASK, NULL},
	GNOMEUIINFO_END
};
*/


/* 
 * global variables
 */

GtkWidget *appbar;
GtkWidget *progress;

GtkWidget *infobar;
guint context_id;

GtkWidget *groups_vbox;
GtkWidget *articlelist_ctree;
GtkWidget *text_box;
GtkWidget *contents_vbox;

int viewmode = 0; /* 1 = paned, 2 = notebook */

GtkWidget *from_label;
GtkWidget *subject_label;
GtkWidget *date_label;



void
gui_select_all (void)
{
	if (viewmode == GUI_PANED)
		gui_paned_select_all ();
	else if (viewmode == GUI_NOTEBOOK)
		gui_notebook_select_all ();
}


/*---[ gui_about ]----------------------------------------------------
 * informative about window popup
 *--------------------------------------------------------------------*/
void
gui_about (void)
{
	static GtkWidget *dialog = NULL;

	if (dialog != NULL)
	{
		g_assert (GTK_WIDGET_REALIZED (dialog));
		gdk_window_show (dialog->window);
		gdk_window_raise (dialog->window);
	}

	else
	{
		const gchar *authors[] = {
			"Matt Eagleson <e.Messiah@superpimp.org>",
			"Jason Leach <elerium@superpimp.org>",
			"Charles Kerr <charles@superpimp.org>",
			NULL
		};

		dialog = gnome_about_new ("Pan", VERSION,
					  _("Copyright e.Messiah (C) 1999"),
					  authors,
					  "The Pimp Ass Newsreader (PAN), a newsreader for GNOME.",
					  "xpm/pan_druid.xpm");
		gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
				    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
				    &dialog);
		if (Pan.window)
			gnome_dialog_set_parent (GNOME_DIALOG (dialog),
						 GTK_WINDOW (Pan.window));

		gtk_widget_show (dialog);
	}
}


void
gui_clean (void)
{
	pan_lock ();
	gnome_appbar_clear_stack (GNOME_APPBAR(appbar));
	if (progress) {
		gtk_progress_set_activity_mode (GTK_PROGRESS(progress), FALSE);
		progress = NULL;
	}
	gnome_appbar_set_progress (GNOME_APPBAR(appbar), 0);
	pan_unlock ();
}


void
gui_create_appbar (GtkWidget* window)
{
	appbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_USER);
	gnome_appbar_set_default (GNOME_APPBAR(appbar), _("Pan - Offline"));
	gnome_app_set_statusbar (GNOME_APP(window), appbar);
}


GtkWidget*
gui_create_infobar (void)
{
	GtkWidget *newbar;

	newbar = gtk_statusbar_new ();
	context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR(newbar), "pan");
	return newbar;
}


void
gui_set_status (gchar *msg)
{
	pan_lock ();
	gnome_appbar_pop (GNOME_APPBAR(appbar));
	gnome_appbar_push (GNOME_APPBAR(appbar), msg);
	pan_unlock ();
}


void
gui_set_status_default (gchar *msg)
{
	pan_lock ();
	gnome_appbar_set_default (GNOME_APPBAR(appbar), msg);
	pan_unlock ();
}


void
gui_set_status_va (const gchar * format, ...)
{
	va_list args;
	gchar *msg;

	g_return_if_fail (format != NULL);

	va_start (args, format);
	msg = g_strdup_vprintf (format, args);
	va_end (args);

	pan_lock ();
	gnome_appbar_pop (GNOME_APPBAR(appbar));
	gnome_appbar_push (GNOME_APPBAR(appbar), msg);
	pan_unlock ();
	g_free (msg);
}


/**********************************************************
 * Progressbar
 **********************************************************/

/*
 * set the progress bar to a percentage
 */
void
gui_set_progress (gfloat percent)
{
	pan_lock ();
	if ((percent >= 0) && (percent <= 1))
		gnome_appbar_set_progress (GNOME_APPBAR(appbar), percent);
	pan_unlock ();
}

static int _step = -1;
static int _steps = -1;
static int _step_last_percent = -1;

void
gui_progress_init_steps ( int steps )
{
	_step = 0;
	_steps = steps;
	_step_last_percent = 0;
	gui_set_progress ( 0.0 );
}


void
gui_progress_next_step (void)
{
	const gfloat percent = _steps ? (++_step / (gfloat)_steps) : 0.0;
	const int step_percent = (int)( 100.0 * percent );
	if ( step_percent != _step_last_percent ) {
		_step_last_percent = step_percent;
		gui_set_progress ( percent );
	}
}

void
gui_progress_next_step_gfunc (gpointer data, 
			      gpointer user_data)
{
	gui_progress_next_step ( );
}



/* ??? */
void
gui_set_progress_activity (gint count)
{
	pan_lock ();
	if (progress == NULL) {
		progress = GTK_WIDGET (gnome_appbar_get_progress (GNOME_APPBAR(appbar)));
		gtk_progress_set_activity_mode (GTK_PROGRESS(progress), TRUE);
		gtk_progress_bar_set_discrete_blocks (GTK_PROGRESS_BAR(progress), 10);
		gtk_progress_bar_set_activity_step (GTK_PROGRESS_BAR(progress), 1);
	}
	gtk_progress_set_value (GTK_PROGRESS(progress), (gfloat) (count % 11));
	pan_unlock ();
}

/**********************************************************
 * 
 **********************************************************/



/*---[ gui_set_info ]-------------------------------------------------
 * set the infobar to a single message
 *--------------------------------------------------------------------*/
void
gui_set_info (gchar* msg)
{
	pan_lock ();
	gtk_statusbar_pop (GTK_STATUSBAR(infobar), context_id);
	gtk_statusbar_push (GTK_STATUSBAR(infobar), context_id, msg);
	pan_unlock ();
}


/*---[ gui_set_info_va ]----------------------------------------------
 * set the infobar to a message (variable arguments)
 *--------------------------------------------------------------------*/
void
gui_set_info_va (const gchar *format, ...)
{
	va_list args;
	gchar *msg;

	g_return_if_fail (format != NULL);

	va_start (args, format);
	msg = g_strdup_vprintf (format, args);
	va_end (args);

	pan_lock ();
	gtk_statusbar_pop (GTK_STATUSBAR(infobar), context_id);
	gtk_statusbar_push (GTK_STATUSBAR(infobar), context_id, msg);
	pan_unlock ();
	g_free (msg);
}


/*---[ gui_shutdown ]-------------------------------------------------
 * save geometry states into config files
 *--------------------------------------------------------------------*/

static void
exit_cb (void)
{
	gui_shutdown ();
	gtk_object_destroy ( GTK_OBJECT(Pan.window) );
}

static int
window_delete_event_cb (GtkWidget *widget,
			GdkEvent *event,
			gpointer  data)
{
	gui_shutdown ();
	return FALSE; /* causes "destroy" signal to be fired */
}


void
gui_shutdown (void)
{
	gint x = 0, y = 0;
	gint width = 640, height = 400;

	gdk_window_get_position (Pan.window->window, &x, &y);
	gdk_window_get_size (Pan.window->window, &width, &height);
				
	gnome_config_set_int ("/Pan/Geometry/pan_x", x);
	gnome_config_set_int ("/Pan/Geometry/pan_y", y);
	gnome_config_set_int ("/Pan/Geometry/width", width);
	gnome_config_set_int ("/Pan/Geometry/height", height);
	gnome_config_set_int ("/Pan/State/viewmode", viewmode);
	gnome_config_set_int ("/Pan/State/group_mode", group_mode);

	if (viewmode == GUI_NOTEBOOK) {
		gnome_config_set_int ("/Pan/State/page", gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)));
	}
	else if (viewmode == GUI_PANED) {
		gnome_config_set_int ("/Pan/Geometry/vpaned", articlelist_ctree->allocation.height);
		gnome_config_set_int ("/Pan/Geometry/hpaned", groups_vbox->allocation.width);
	}

	gui_save_column_widths (GTK_CLIST (Pan.group_clist), "group");
	gui_save_column_widths (GTK_CLIST (Pan.article_ctree), "articlelist");

	gnome_config_sync();
}


/*---[ gui_construct ]------------------------------------------------
 * initialize and place the basic interface components
 *--------------------------------------------------------------------*/
void
gui_construct (const gchar *geometry)
{
	GtkWidget *w, *x;
	GtkWidget *hbox;
//	GtkStyle *boldstyle;
	/* int geo_x=0, geo_y=0; */
	GtkTooltips *ttips;

	ttips = gtk_tooltips_new();

	Pan.window = gnome_app_new ("Pan", "Pan");
	gtk_signal_connect (GTK_OBJECT(Pan.window), "delete_event", 
			    GTK_SIGNAL_FUNC (window_delete_event_cb), NULL);
	gtk_signal_connect (GTK_OBJECT(Pan.window), "destroy",//delete_event", 
			    GTK_SIGNAL_FUNC (pan_shutdown), NULL);

	gtk_widget_realize (Pan.window);

	/** geometry parsing **/
	/* The following chunk of code will allow command line arguments
	 * to override the config. So if this was invoked as part of a 
	 * session we 'Do the right thing' :)
	 */
	if (geometry != NULL)
	  {
	    gint x, y, w, h;
	    if (gnome_parse_geometry (geometry, &x, &y, &w, &h) )
	      {
		if (x != -1)
		  gtk_widget_set_uposition (Pan.window, x, y);
		if (w != -1)
		  gtk_window_set_default_size (GTK_WINDOW (Pan.window), w, h);
	      }
	    else
	      {
		g_error (_("Unable to parse the geometry string '%s'"), geometry);
	      }
	  }
	else
	  {                  /* Lets see if the config has a geometry */
	    gint w, h;

	    w = gnome_config_get_int ("/Pan/Geometry/width");
	    h = gnome_config_get_int ("/Pan/Geometry/height");
	    if (w != 0 && h != 0)
		    gtk_window_set_default_size (GTK_WINDOW (Pan.window), w, h);
	    else
		    gtk_window_set_default_size (GTK_WINDOW (Pan.window), 640, 400);

/* While this is nice for some people, it's typically much better to let
 * the window manager do the position saving and restoring... we have similar
 * code at the bottom of this function...

	    x = gnome_config_get_int ("/Pan/Geometry/pan_x");
	    y = gnome_config_get_int ("/Pan/Geometry/pan_y");
	    if (x != 0 && y != 0)
	      gtk_widget_set_uposition (Pan.window, x, y);
*/
	  }

/* Groups vbox */
	groups_vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_set_usize (GTK_WIDGET (groups_vbox), 300, 200);
	x = gtk_hbox_new (FALSE, 0);

	gtk_box_pack_start (GTK_BOX (x), gtk_label_new (_("Find: ")), FALSE, FALSE, 3);
	w = gtk_entry_new ();
	gtk_signal_connect (GTK_OBJECT (w), "activate",
			    GTK_SIGNAL_FUNC(grouplist_grep), w);

	gtk_tooltips_set_tip (GTK_TOOLTIPS (ttips), w,
			      _("Type in a search string and press ENTER.  Leave this blank and hit ENTER to get a list of all groups"),
			      _("If there are no groups listed, leave this blank and press ENTER"));
//	gtk_tooltips_enable (GTK_TOOLTIPS (ttips));

	gtk_box_pack_start (GTK_BOX (x), w, TRUE, TRUE, 3);

	gtk_box_pack_start (GTK_BOX (groups_vbox), grouplist_create (), TRUE, TRUE, 1);
	gtk_box_pack_start (GTK_BOX (groups_vbox), x, FALSE, FALSE, 1);

/* Articlelist_ctree */
	articlelist_ctree = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (articlelist_ctree), create_articlelist_ctree (), TRUE, TRUE, 0);

/* Text table */
	text_box = gtk_vbox_new (FALSE, 0);

	hbox = gtk_hbox_new (FALSE, 0);
	from_label = gtk_label_new (NULL);
	w = gtk_label_new (_("From: "));
//	boldstyle = gtk_style_new ();
//	boldstyle->font = gdk_font_load ("-b&h-lucida-bold-r-normal-*-*-120-*-*-p-*-iso8859-1");
	
//	gtk_widget_set_style (GTK_WIDGET (w), boldstyle);
	gtk_misc_set_alignment (GTK_MISC (from_label), 0.0, 0.5);

	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (hbox), from_label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (text_box), hbox, FALSE, FALSE, 0);

	hbox = gtk_hbox_new (FALSE, 0);
	subject_label = gtk_label_new (NULL);
	w = gtk_label_new (_("Subject: "));
//	gtk_widget_set_style (GTK_WIDGET (w), boldstyle);
	gtk_misc_set_alignment (GTK_MISC (subject_label), 0.0, 0.5);

	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (hbox), subject_label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (text_box), hbox, FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (text_box), text_create(), TRUE, TRUE, 0);

	gui_create_appbar (Pan.window);

	gnome_app_create_menus (GNOME_APP (Pan.window), pan_main_menu);
//	gnome_app_create_toolbar (GNOME_APP (Pan.window), pan_main_toolbar);
	gnome_app_install_menu_hints (GNOME_APP (Pan.window), pan_main_menu);

	contents_vbox = gtk_vbox_new (FALSE, 0);
	gnome_app_set_contents (GNOME_APP (Pan.window), contents_vbox);

	/* connect signals */
        gtk_signal_connect (GTK_OBJECT (Pan.article_ctree), "key_press_event",
                            GTK_SIGNAL_FUNC (gui_key_press_cb), NULL);
        gtk_signal_connect (GTK_OBJECT (Pan.group_clist), "key_press_event",
                            GTK_SIGNAL_FUNC (gui_key_press_cb),  NULL);
        gtk_signal_connect (GTK_OBJECT (text), "key_press_event",
                            GTK_SIGNAL_FUNC (gui_key_press_cb), NULL);

        gui_set_layout ( gnome_config_get_int ( "Pan/State/viewmode=1" ) );

/* FIXME: this isn't useful until we can get it to automatically load the last
   selected group or message.  Otherwise, it remembers you were in the article
   list view, but then it's just blank, so you gotta switch back to the group
   list mode anyhow.
   if (viewmode == GUI_NOTEBOOK) {
   int last_page = gnome_config_get_int ("/Pan/State/page=0");
   tk_notebook_set_page (GTK_NOTEBOOK (notebook), last_page);
   }
*/

	/* FIXME: 0,0 shows up under the panel for me on IceWm
	 * I'm not sure we should mess with window placement
	 * until the user tells us to... 
	 geo_x = gnome_config_get_int ("/Pan/Geometry/pan_x=0");
	 geo_y = gnome_config_get_int ("/Pan/geometry/pan_y=0");
	 gdk_window_move (Pan.window->window, geo_x, geo_y);
	*/
}


/*---[ replace_checkbox ]---------------------------------------------
 * Update one of our homebrew checkboxes.
 * I copied the idea from the groups menu.
 * (We're using homebrew checkboxes to get the pretty checkbox icon?)
 * FIXME: if gnome user sets /Gnome/Icons/MenusUseIcons to false, this fails..
 *--------------------------------------------------------------------*/
static void
replace_checkbox ( const GnomeUIInfo* e, int is_checked, const gchar* menu_path )
{
	GnomeUIInfo entry[2];
	entry[0] = *e;
	if ( is_checked )
	{
		entry[0].pixmap_type = GNOME_APP_PIXMAP_STOCK;
		entry[0].pixmap_info = GNOME_STOCK_PIXMAP_CLOSE;
	}
	else
	{
		entry[0].pixmap_type = GNOME_APP_PIXMAP_NONE;
		entry[0].pixmap_info = NULL;
	}
	entry[1].type = GNOME_APP_UI_ENDOFINFO;
	gnome_app_insert_menus ( GNOME_APP(Pan.window), menu_path, entry );
	gnome_app_remove_menus ( GNOME_APP(Pan.window), menu_path, 1 );
}




/*---[ gui_layout ]---------------------------------------------------
 * main menu callback
 * gui_set_layout (layout)
 *--------------------------------------------------------------------*/
static void
layout_cb ( GtkButton* button )
{
   gui_set_layout ( (int) gtk_object_get_data ( GTK_OBJECT(button), GNOMEUIINFO_KEY_UIDATA ) );
}

void
gui_set_layout ( int new_layout )
{
	const int size = 2;
	int layouts [] = { GUI_PANED, GUI_NOTEBOOK };
	gchar* text [] = { N_(layout_paned_text), N_(layout_notebook_text) };
	gchar* hint [] = { N_(layout_paned_hint), N_(layout_notebook_hint) };
	int i;

	/* arg sanity check */
	g_return_if_fail ( new_layout==GUI_PANED || new_layout==GUI_NOTEBOOK );

	/* rebuild the homebrew radiobutton menu items */
	for ( i=size-1; i>=0; --i )
	{
		gchar* path = g_strjoin ( "/", "View", text[i], NULL );

		GnomeUIInfo entry;
		entry.type = GNOME_APP_UI_ITEM;
		entry.label = text[i];
		entry.hint = hint[i];
		entry.moreinfo = (gpointer)layout_cb;
		entry.user_data = (gpointer)layouts[i];
		entry.unused_data = NULL;
		replace_checkbox ( &entry, new_layout==layouts[i], path );

		g_free ( path );
	}

	/* build the desired layout */
	switch ( new_layout ) {
		case GUI_PANED: gui_paned_construct ( ); break;
		case GUI_NOTEBOOK: gui_notebook_construct ( ); break;
	}
}


/*---[ gui_page_set ]-------------------------------------------------
 * flip to a different notebook page, setting keyboard focus on a 
 * widget in the new page
 *--------------------------------------------------------------------*/
void
gui_page_set (int page, GtkWidget *focus_item)
{
	if (viewmode == GUI_NOTEBOOK) {
		gtk_notebook_set_page (GTK_NOTEBOOK (notebook), page);
		gtk_widget_grab_focus (focus_item);
	}
}


/*---[ gui_page_change ]----------------------------------------------
 * flip to a differnt notebook page, setting keyboard focus in new page.  
 * This is different from gui_page_set because it flips the page based on 
 * a offset, like -1 or 1, to flip back one page or forward one.  It is 
 * automatic in handling the focus grabbing too.
 *--------------------------------------------------------------------*/
void
gui_page_change (int change)
{
	int cur_page;
	group_data* gdata;

	if (viewmode == GUI_PANED)
		cur_page = gui_paned_get_current_pane ();
	if (viewmode == GUI_NOTEBOOK)
		cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
	else
		return;

	cur_page += change;
	if ((cur_page < 0) || (cur_page > 2)) 
		return;

	if (change < 0) { /* Escape was pressed */
		switch (cur_page) {
		case GROUPS_PAGE:
			gtk_widget_grab_focus (Pan.group_clist);
			break;
		case ARTICLELIST_PAGE:
			gtk_widget_grab_focus (Pan.article_ctree);
			break;
		case MESSAGE_PAGE:
			gtk_widget_grab_focus (text);
			break;
		default:
			break;
		}
	}
	else { /* enter key hit */
/* FIXME: segfaults if selection is zero, but when user hits escape, it
   automatically sends a "undo_selection" signal, deselecting all */
		switch (cur_page) {
		case GROUPS_PAGE:
			gtk_widget_grab_focus (Pan.group_clist);
			break;
		case ARTICLELIST_PAGE:
//			if (!GTK_CLIST (Pan.group_clist)->selection && !GTK_CLIST (Pan.group_clist)->undo_selection)
//				return;

			gdata = (group_data *) gtk_clist_get_row_data (GTK_CLIST (Pan.group_clist), 
								       GPOINTER_TO_INT (GTK_CLIST (Pan.group_clist)->selection->data));

			if ( gdata )
				group_set_active_group ( gdata );

			gtk_widget_grab_focus (Pan.article_ctree);
			break;
		case MESSAGE_PAGE:
		{
			const gchar* message_id = (gchar*) gtk_ctree_node_get_row_data (
				GTK_CTREE (Pan.article_ctree),
				GTK_CLIST (Pan.article_ctree)->selection->data );
			if ( message_id ) {
				text_set_from_message_id ( message_id );
				gtk_widget_grab_focus ( text );
			}

			break;
		}
		default:
			break;
		}
	}

	if (viewmode == GUI_NOTEBOOK)
		gtk_notebook_set_page (GTK_NOTEBOOK (notebook), cur_page);
/*
  FIXME: this would be nice, there is a slight problem with the tabs at the top
  being refreshed after repeated flipping around.  But redrawing the entire
  notebook looks flickery and will waste a lot of resources.  Maybe redraw at
  some other less frequent place or redraw only the tab elements (if that is 
  possible).
  gtk_widget_draw (GTK_WIDGET (notebook), NULL);
*/
}


static
void
gui_enter_clicked ( void )
{
	if (viewmode == GUI_NOTEBOOK)
		gui_page_change (1);
	else if (viewmode == GUI_PANED)
		gui_paned_enter_clicked ();
}


/*---[ gui_set_title ]------------------------------------------------
 * set Pan's title to reflect the current open group
 *--------------------------------------------------------------------*/
void
gui_set_title (group_data *gdata)
{
	gchar *buf;
	
	if (gdata)
		buf = g_strdup_printf ("Pan [%s]", gdata->name);
	else
		buf = g_strdup_printf ("Pan %s", VERSION );

	gtk_window_set_title (GTK_WINDOW (Pan.window), buf);
	g_free (buf);

}

/*---[ gui_key_press_cb ]---------------------------------------------
 * key press callback, when a key is pressed in the group list, article
 * list, or in the message view.  This allows better keyboard navigation
 * and the potential for new key bindings to imitate Ag*nt.
 *--------------------------------------------------------------------*/
void
gui_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
	switch (event->keyval)
	{
	case GDK_Q:
	case GDK_q:
		gui_page_change (-1);
		break;
	case GDK_Return:
//		gui_page_change (1);
		gui_enter_clicked ();
		break;
	default:
		break;
	}
}


/*---[ gui_popup_draw ]----------------------------------------------
 * use this to draw popup windows or dialogs, it will center them in the 
 * exact center of the Pan window, show, and raise them.
 *--------------------------------------------------------------------*/
void
gui_popup_draw (GtkWidget *dialog)
{
	int x=0, y=0, width=640, height=400;
	int dlg_width, dlg_height;
	int dlg_x, dlg_y;

/*	gtk_window_position (GTK_WINDOW (dialog), GTK_WIN_POS_NONE); */
	
	gdk_window_get_position (Pan.window->window, &x, &y);
	gdk_window_get_size (Pan.window->window, &width, &height);

	gtk_widget_realize (dialog);
	gdk_window_get_size (dialog->window, &dlg_width, &dlg_height);

	dlg_x = x + width/2 - dlg_width/2;
	dlg_y = y + height/2 - dlg_height/2;

	gtk_widget_set_uposition (dialog, dlg_x, dlg_y);
	gtk_widget_show_all (dialog);

	gdk_window_raise (dialog->window);	
}


void
gui_restore_column_widths (GtkCList *clist, gchar *type)
{
	int i;
	int tmp_width;
	int cols = GTK_CLIST (clist)->columns;

	for (i = 0; i < cols; i++)
	{
		gchar* buf = NULL;
		if (i == 0)
			buf = g_strdup_printf ( "/Pan/Geometry/%s_column_0=200", type);
		else
			buf = g_strdup_printf ( "/Pan/Geometry/%s_column_%d", type, i);
		tmp_width = gnome_config_get_int (buf);
		g_free ( buf );

		gtk_clist_set_column_width (GTK_CLIST (clist), i, tmp_width);
	}
}


void
gui_save_column_widths (GtkCList *clist, gchar *type)
{
	int i;
	int cols = GTK_CLIST (clist)->columns;
	
	for (i = 0; i < cols; i++) {
		gchar* buf = g_strdup_printf ( "/Pan/Geometry/%s_column_%d", type, i );
		gnome_config_set_int (buf, (int) GTK_CLIST (clist)->column[i].width);
		g_free ( buf );
	}
}
