/*
 * Pan - A Newsreader for X
 * Copyright (c) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef __GUI_H__
#define __GUI_H__

extern GtkWidget *appbar;
extern GtkWidget *progress;

extern GtkWidget *infobar;
extern GtkWidget *groups_vbox;
extern GtkWidget *articlelist_ctree;
extern GtkWidget *text_box;
extern GtkWidget *contents_vbox;
extern GtkWidget *text;

extern int viewmode;

extern GtkWidget *from_label;
extern GtkWidget *subject_label;
extern GtkWidget *date_label;


extern GnomeUIInfo help_menu[];

void gui_clean (void);
void gui_set_status_default (gchar *);
void gui_set_status (gchar *);
void gui_set_status_va (const gchar *, ...);
void gui_set_progress (gfloat);
void gui_set_progress_activity (int);
void gui_set_info (gchar *);
void gui_set_info_va (const gchar *, ...);
void gui_set_layout(int);
void gui_about (void);
void gui_shutdown (void);
void gui_show_unsubscribed_groups (int);

/**
 * Prepare the progressbar for calls to gui_progress_next_step().
 * This is preferred over gui_set_progress for multistep items
 * because it lowers the number of times the gui has to be locked.
 *
 * @param step_qty number of times gui_progress_next_step() will
 * be called while the statusbar goes from 0 to 100%.
 */ 
extern void gui_progress_init_steps ( int total_step_count );

/**
 * Advance the progress meter one step.
 * For more infom, see gui_progress_init_steps().
 */
extern void gui_progress_next_step ( void );

extern void gui_progress_next_step_gfunc ( gpointer, gpointer );


enum {
	GUI_PANED = 1,
	GUI_NOTEBOOK
};

void gui_construct (const gchar *geometry);

enum {
	GROUPS_PAGE = 0,
	ARTICLELIST_PAGE,
	MESSAGE_PAGE
};

void gui_page_set (int page, GtkWidget *focus_item);
void gui_page_change (int change);

void gui_key_press_cb (GtkWidget* widget, GdkEventKey* event, gpointer data);

void gui_set_title (group_data *gdata);

void gui_popup_draw (GtkWidget *dialog);

GtkWidget* gui_create_infobar (void);
void gui_create_appbar (GtkWidget* window);

void gui_restore_column_widths (GtkCList *clist, gchar *type);
void gui_save_column_widths    (GtkCList *clist, gchar *type);

void gui_select_all (void);

#endif /* __GUI_H__ */
