/*
 * Pan - A Newsreader for GNOME
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

// logging extension  for pan
// (c) 1999 Jason Leach
// released under the GNU GPL

// key:
// ~ = partially coded
// ! = fully coded (I believe), but not tested
// * = finished and tested

// extras (add last if at all):
//
// [ ] ability to choose logging options (downloaded files/connections/group updates)
// [ ] ability to limit max log size
// [ ] colored log
// [ ] log viewer editing options (remove selected...etc)

#include <time.h>
#include <string.h>
#include "pan.h"

PanLogViewer *pan_log_viewer;

static GnomeUIInfo log_file_menu[] = {
//	GNOMEUIINFO_MENU_OPEN_ITEM (not_implemented, NULL),
//	GNOMEUIINFO_MENU_SAVE_ITEM (log_save_cb, NULL),
	GNOMEUIINFO_MENU_SAVE_AS_ITEM (log_save_cb, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_PRINT_ITEM (not_implemented, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_CLOSE_ITEM (log_hide_cb, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo log_edit_menu[] = {
//	GNOMEUIINFO_MENU_CUT_ITEM (log_cut_cb, NULL),
	GNOMEUIINFO_MENU_COPY_ITEM (log_copy_cb, NULL),
//	GNOMEUIINFO_MENU_PASTE_ITEM (log_paste_cb, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo log_view_menu[] = {
	GNOMEUIINFO_END
};

static GnomeUIInfo log_main_menu[] = {
	GNOMEUIINFO_MENU_FILE_TREE (log_file_menu),
	GNOMEUIINFO_MENU_EDIT_TREE (log_edit_menu),
	GNOMEUIINFO_SUBTREE(N_("_View"), &log_view_menu),
	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
	GNOMEUIINFO_END
};

static GnomeUIInfo log_toolbar[] = {
	GNOMEUIINFO_ITEM_STOCK (N_("Save Log"), N_("Save this log to a file."), 
				log_save_cb, GNOME_STOCK_PIXMAP_SAVE),
	GNOMEUIINFO_ITEM_STOCK (N_("Clear Log"), N_("Clear this log."),
				log_clear_cb, GNOME_STOCK_PIXMAP_CLEAR),
	GNOMEUIINFO_ITEM_STOCK (N_("Close"), N_("Close the Log Viewer."),
				log_hide_cb, GNOME_STOCK_PIXMAP_CLOSE),
	GNOMEUIINFO_END

};


void
log_add (gchar *msg)
{
	time_t cur_time;
	gchar datestr[256];
	gchar *data[2]; // = { datestr, msg };

	time (&cur_time);

	g_snprintf (datestr, 256, "%s", ctime(&cur_time));
	remove_crlf (datestr);

	data[0] = (gchar *) g_malloc0 (256);
	data[1] = (gchar *) g_malloc0 (256);

	g_snprintf (data[0], 256, "%s", msg);
	g_snprintf (data[1], 256, "%s", datestr);

	pan_lock();
	gtk_clist_freeze (GTK_CLIST(pan_log_viewer->log));
	gtk_clist_append (GTK_CLIST(pan_log_viewer->log), data);
	gtk_clist_thaw (GTK_CLIST(pan_log_viewer->log));
	pan_unlock ();

	g_free (data[0]);
	g_free (data[1]);

}


/*---[ log_add ]------------------------------------------------------
 * add a message to the log file
 *--------------------------------------------------------------------*/
#if 0
void
log_add_old (gchar *msg)
{
	time_t cur_time;
	char log_entry[256];

	time (&cur_time);

	g_snprintf (log_entry, 256, "%s", ctime(&cur_time));
	remove_crlf (log_entry);

	strcat (log_entry, ": ");
	strcat (log_entry, msg);
	strcat (log_entry, "\n");

	pan_lock();
	gtk_text_freeze (GTK_TEXT(pan_log_viewer->log));
	gtk_text_set_point (GTK_TEXT(pan_log_viewer->log), 0);
	gtk_text_insert (GTK_TEXT(pan_log_viewer->log), NULL, NULL, NULL, log_entry, -1);
	gtk_text_thaw (GTK_TEXT(pan_log_viewer->log));
	pan_unlock();
}
#endif


/*---[ log_add_va ]---------------------------------------------------
 * add a message to the log file with a variable argument list
 *--------------------------------------------------------------------*/
void
log_add_va (gchar *format, ...)
{
	va_list args;
	gchar *msg;

	g_return_if_fail (format != NULL);

	va_start (args, format);
	msg = g_strdup_vprintf (format, args);
	va_end (args);

	log_add (msg);
	g_free (msg);
}


void
log_copy_cb (GtkWidget *widget, PanLogViewer *plw)
{
	gtk_editable_copy_clipboard (GTK_EDITABLE (plw->log));
}


/*---[ log_clear_cb ]-------------------------------------------------
 * erase the log widget
 *--------------------------------------------------------------------*/
void
log_clear_cb (GtkWidget *widget, PanLogViewer *plw)
{
	pan_lock ();
	gtk_clist_freeze (GTK_CLIST (plw->log));
	gtk_clist_clear (GTK_CLIST (plw->log));
	gtk_clist_thaw (GTK_CLIST (plw->log));
	pan_unlock ();
}


/* FIXME: update to use the new clist */
void
log_cut_cb (GtkWidget *widget, PanLogViewer *plw)
{
	gtk_editable_cut_clipboard (GTK_EDITABLE(plw->log));
}


/*---[ log_hide_cb ]--------------------------------------------------
 * this callback hides the window...
 *--------------------------------------------------------------------*/
void
log_hide_cb (GtkWidget *widget, PanLogViewer *lw)
{
	gtk_widget_hide (pan_log_viewer->window);
}


/*---[ log_init ]-----------------------------------------------------
 * initialization stuff, called from pan_init()
 *--------------------------------------------------------------------*/
void
log_init (void)
{
	gchar *titles[2] = {
		_("Log Entry"),
		_("Time")
	};

	pan_log_viewer = (PanLogViewer *) g_malloc0 (sizeof(PanLogViewer));

	pan_log_viewer->log = gtk_clist_new_with_titles (2, titles);
	gtk_clist_set_column_auto_resize (GTK_CLIST(pan_log_viewer->log), 0, TRUE);
	gtk_clist_set_column_auto_resize (GTK_CLIST(pan_log_viewer->log), 1, TRUE);

}


/* FIXME: update for the new clist */
void
log_paste_cb (GtkWidget *widget, PanLogViewer *plw)
{
	gtk_editable_paste_clipboard (GTK_EDITABLE (plw->log));
}


/*--------------------------------------------------------------------
 *
 *--------------------------------------------------------------------*/
void
log_save_cb (GtkWidget *widget, PanLogViewer *lw)
{
	lw->file_entry = gtk_file_selection_new (_("Save the log to file"));

	gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (lw->file_entry));

	gtk_signal_connect_object (GTK_OBJECT (lw->file_entry), "destroy",
				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
				   GTK_OBJECT (lw->file_entry));
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (lw->file_entry)->cancel_button),
				   "clicked",
				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
				   GTK_OBJECT (lw->file_entry));
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (lw->file_entry)->ok_button),
			    "clicked",
			    GTK_SIGNAL_FUNC (log_save_ok_clicked),
			    lw);

	gtk_widget_show_all (lw->file_entry);
}


void
log_save_ok_clicked (GtkWidget *widget, PanLogViewer *plw)
{
	gchar *filename;
	FILE *log_dump;
	gint tot_rows, i;
	gchar *data[] = { NULL, NULL };
	gchar *log_text;

	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (plw->file_entry));

/*	g_print ("%s\n", filename); */

	gtk_clist_freeze (GTK_CLIST (plw->log));
	tot_rows = gtk_clist_append (GTK_CLIST (plw->log), data);
	gtk_clist_remove (GTK_CLIST (plw->log), tot_rows);
	gtk_clist_thaw (GTK_CLIST (plw->log));

	log_text = g_malloc0 (256);

	if ((log_dump = fopen (filename, "w")) == NULL) {
		g_print ("fopen() failed to save log");
		return;
	}
	g_free(filename);

	for (i = 0; i < tot_rows; i++) {
		gtk_clist_get_text (GTK_CLIST(plw->log), i, 0, &log_text);
		strcat (log_text, " ");
		fputs (log_text, log_dump);	

		gtk_clist_get_text (GTK_CLIST(plw->log), i, 1, &log_text);
		fputs (log_text, log_dump);
	}

	fclose (log_dump);
	g_free (log_text);

	gtk_widget_destroy (GTK_WIDGET (plw->file_entry));
}


void
log_setup_appbar (void)
{
	pan_log_viewer->appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
	gnome_app_set_statusbar (GNOME_APP(pan_log_viewer->window), GTK_WIDGET (pan_log_viewer->appbar));
}


/*--------------------------------------------------------------------
 * pop open a log viewer, use a generic scrolling window with a
 * text box inside to show the log of past events
 *--------------------------------------------------------------------*/
void
log_viewer_spawn (void)
{
	GtkWidget *scrolled_window;

	if (GTK_IS_WIDGET (pan_log_viewer->window)) {
		gtk_widget_show (GTK_WIDGET (pan_log_viewer->window));
		gdk_window_raise (pan_log_viewer->window->window);
		return;
	}

	pan_log_viewer->window = gnome_app_new ("Log", _("Pan - Log Viewer"));

	scrolled_window = gtk_scrolled_window_new (NULL, NULL);

	log_setup_appbar ();

	gnome_app_set_contents (GNOME_APP (pan_log_viewer->window), scrolled_window);
	gnome_app_create_menus_with_data (GNOME_APP (pan_log_viewer->window), log_main_menu, pan_log_viewer);
	gnome_app_create_toolbar_with_data (GNOME_APP (pan_log_viewer->window), log_toolbar, pan_log_viewer);
	gnome_app_install_menu_hints (GNOME_APP(pan_log_viewer->window), log_main_menu);

	gtk_window_set_default_size (GTK_WINDOW (pan_log_viewer->window), 450, 336);

	gtk_signal_connect_object (GTK_OBJECT (pan_log_viewer->window), "delete_event",
				   GTK_SIGNAL_FUNC (log_hide_cb),
				   GTK_OBJECT (pan_log_viewer->window));

	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);

	gtk_container_add (GTK_CONTAINER (scrolled_window), pan_log_viewer->log);

	gtk_widget_show_all (pan_log_viewer->window);
}


/*--------------------------------------------------------------------
 * uhh, this isn't used yet
 *--------------------------------------------------------------------*/
void
log_update (void)
{
/*
	guint pos;
	gchar *tmp;
	
	pos = gtk_text_get_length (GTK_TEXT (plw->log));
	gtk_text_set_point (GTK_TEXT (plw->log), pos);
	gtk_text_backward_delete (GTK_TEXT (plw->log), pos);

	gtk_editable_delete_chars (GTK_TEXT (plw->log), 0, -1);
	tmp = gtk_editable_get_chars (GTK_TEXT (plw->log), 0, -1);
*/
}


/*--------------------------------------------------------------------
 * remove CRLF (newline) from a string
 *--------------------------------------------------------------------*/
void
remove_crlf (gchar *msg)
{
	char *tmp;
	if (!msg)
		return;

	while ((tmp = strrchr(msg, '\n')) || (tmp = strrchr(msg, '\r')))
		*tmp = 0;
}
