/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "pan.h"

PanApp Pan;

static gchar     *geometry = NULL;
poptContext      pctx;
struct poptOption options[] = {
	{ "geometry", '\0', POPT_ARG_STRING, &geometry, 0,
	  N_("Specify the geometry of the window."),
	  N_("GEOMETRY") },
	{ NULL, '\0', 0, NULL, 0 }
};


/*---[ pan_init ]-----------------------------------------------------
 * pan_init() is when the druid finishes on Pan's first run, after 
 * that it is called right before gtk_main() is called
 *--------------------------------------------------------------------*/
void
pan_init (void)
{
	log_init ();
	server_init ();
	queue_init ();
	nntp_init ();
//	queue_manager_init ();
}


/*---[ main ]---------------------------------------------------------
 * _the_ main function!
 *--------------------------------------------------------------------*/
int
main (int argc, char *argv[])
{
        GnomeClient *client;

	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
	textdomain(PACKAGE);

	g_thread_init (NULL);

	gnome_init_with_popt_table ("Pan", VERSION, argc, argv, options, 0, &pctx);
	poptFreeContext (pctx);
	prefs_init ();

	/* Connect up to the session manager */
	client = gnome_master_client ();
	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
			    GTK_SIGNAL_FUNC (save_session), argv[0]);
	gtk_signal_connect (GTK_OBJECT (client), "die",
			    GTK_SIGNAL_FUNC (session_die), NULL);

	/* initialize variables in the global app structure */
	Pan.grouplist = NULL;
	Pan.active_group = NULL;
/*	Pan.error = NULL; */

	gui_construct (geometry);

	/* Check if a Preferences file exists. */
	if (!gnome_config_has_section ("/Pan/User")) {
		gtk_widget_show_all (Pan.window);
		druid_spawn ();
	}
	else {
		pan_init ();
		gtk_widget_show_all (Pan.window);
	}

	gdk_threads_enter ();
	gtk_main ();

	return 0;
}


/*---[ pan_shutdown ]-------------------------------------------------
 * shutdown Pan, closing all tcp, and saving all proper config
 * options
 *--------------------------------------------------------------------*/
void
pan_shutdown (void)
{
	tcp_close ();

	gui_set_status (_("Closing article database..."));
	group_set_active_group ( NULL );

	if (Pan.grouplist)
		grouplist_save (Pan.grouplist);

	g_free (message_body_font);
	g_free (articlelist_font);
	g_free (grouplist_font);

	gtk_main_quit ();
}


/*** Session management **/
/**
 * save_session:
 *
 * Saves the current session for later revivial.
 **/
gint
save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
              gint is_shutdown, GnomeInteractStyle interact_style,
              gint is_fast, gpointer client_data)
{
	gchar **argv;
	guint argc;

	argv = g_malloc0(sizeof (gchar *)*4);
	argc = 1;

	argv[0] = client_data;
	/* add any addtional state info here. */
	gnome_client_set_clone_command (client, argc, argv);
	gnome_client_set_restart_command (client, argc, argv);
	return TRUE;
}


/*---[ session_die ]--------------------------------------------------
 *
 *--------------------------------------------------------------------*/
void
session_die (GnomeClient *client, gpointer client_data)
{
	gtk_main_quit ();
}
