/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifdef __FreeBSD__
#include <unistd.h>
#include <netinet/in.h>
#include <sys/socket.h>
#endif

#include <arpa/inet.h>
#include <netdb.h>
#include <pwd.h>
#include "pan.h"
#include "../config.h"


/* slowaris needs sys/socket.h for AF_INET... */
#ifdef AF_NEEDS_SYS_SOCKET_H
#include <sys/socket.h>
#endif


static GnomeUIInfo post_file_menu[] = {
	GNOMEUIINFO_MENU_OPEN_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_SAVE_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_SAVE_AS_ITEM (not_implemented, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_PRINT_ITEM (not_implemented, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM (close_window, NULL),	
	GNOMEUIINFO_END
};

static GnomeUIInfo post_edit_menu[] = {
	GNOMEUIINFO_MENU_CUT_ITEM (post_cut_cb, NULL),
	GNOMEUIINFO_MENU_COPY_ITEM (post_copy_cb, NULL),
	GNOMEUIINFO_MENU_PASTE_ITEM (post_paste_cb, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo post_message_menu[] = {
	{ GNOME_APP_UI_ITEM, N_("_Send Now"), N_("Send this message now."), send_post },
	GNOMEUIINFO_END
};

static GnomeUIInfo post_main_menu[] = {
	GNOMEUIINFO_MENU_FILE_TREE (post_file_menu),
	GNOMEUIINFO_MENU_EDIT_TREE (post_edit_menu),
	GNOMEUIINFO_SUBTREE (N_("_Message"), &post_message_menu),
	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
	GNOMEUIINFO_END
};

static GnomeUIInfo post_toolbar[] = {
	GNOMEUIINFO_ITEM_STOCK (N_("Post"), N_("Post this message"), 
				send_post, GNOME_STOCK_PIXMAP_MAIL_SND),
	GNOMEUIINFO_ITEM_STOCK (N_("Spelling"), N_("Check spelling"),
				NULL, GNOME_STOCK_PIXMAP_SPELLCHECK),
	GNOMEUIINFO_ITEM_STOCK (N_("Print"), N_("Print this message"),
				NULL, GNOME_STOCK_PIXMAP_PRINT),
	GNOMEUIINFO_ITEM_STOCK (N_("Cancel"), N_("Cancel this message"),
				close_window, GNOME_STOCK_PIXMAP_CLOSE),
	GNOMEUIINFO_END
};



void
post_cut_cb (GtkWidget *widget, PostWindow *pw)
{
	gtk_editable_cut_clipboard (GTK_EDITABLE(pw->text));
}


void
post_copy_cb (GtkWidget *widget, PostWindow *pw)
{
	gtk_editable_copy_clipboard (GTK_EDITABLE(pw->text));
}


void
post_paste_cb (GtkWidget *widget, PostWindow *pw)
{
	gtk_editable_paste_clipboard (GTK_EDITABLE(pw->text));
}


void
destroy_post_window (GtkWidget *widget, PostWindow *pw)
{
	gtk_widget_destroy (pw->newsgroups);
	gtk_widget_destroy (pw->subject);
	gtk_widget_destroy (pw->text);
	g_free (pw);
}


void
close_window (GtkWidget *widget, PostWindow *pw)
{
	gtk_widget_destroy (pw->window);
}


gint
check_from_syntax (gchar *s)
{
	return 1;
}


gint
check_newsgroups_syntax (gchar *s)
{
	return 1;
}


static gchar*
generate_references (PostWindow *pw)
{
#define GNKSA_LENGTH 998
	gchar buf[GNKSA_LENGTH + 1];
	gchar *p = NULL;
	gchar *refs = NULL;
	gchar *new_refs = NULL;
	gchar *pch = NULL;
	unsigned int total_len, len;

//	p = extract_header (pw->article, "References");
//	if (p)
//		refs = g_strconcat (p, " ", extract_header(pw->article, "Message-ID"), NULL);
//	else
//		refs = g_strconcat (extract_header(pw->article, "Message-ID"), NULL);
//	g_free (p);
	
	if (pw->article->references) {
		refs = g_strconcat (pw->article->references, " ", Pan.active_message_id, NULL);
	}
	else if (Pan.active_message_id) {
		refs = g_strconcat (Pan.active_message_id, NULL);
	}
	else {
		refs = g_strdup ( "" );
	}



	len = strlen (refs);
	if (len <= GNKSA_LENGTH) {
		return refs;
	}

	/* The reference header is too long so start truncating */
	/* We have to preserve the first message-id */
	p = strchr(refs, '>');
	if (p == NULL)
		return NULL;
	p++;
	len = p - refs;
	if (len >= GNKSA_LENGTH) {
		return refs;
	}
	strncpy(buf, refs, len);
	buf[len] = 0;
	new_refs = g_strconcat(buf, " ", NULL);
	total_len = len + 1;

	refs += len;
	if ((*refs == '\t') || (*refs == ' '))
		refs++;					/* already handled space */

	/* Now use message-ids at end of the line */
	p = refs + strlen(refs);

	while (1) {
		if (*p == '<') {
			if (strlen(p) + total_len > GNKSA_LENGTH) {
				p = strchr(p + 1, '<');
				if (p == NULL)
					p = refs;
				break;
			}
		}
		if (p == refs)
			break;
		p--;
	}

	pch = new_refs;
	new_refs = g_strconcat (new_refs, p, NULL);
	g_free ( pch );
	g_free ( refs );
	return new_refs;
}

static gchar*
generate_message_id (void)
{
	unsigned long now;
	unsigned long randy;
	static unsigned char baseid[64];
	gchar *b, *t, tmp[32];
	char *chars32 = "0123456789abcdefghijklmnopqrstuv";
	static unsigned long last_now;

	while (1) {
		time((time_t *) & now);
		if (now != last_now)
			break;
		sleep (1);
	}
	last_now = now;
	srand (now + getpid());
	randy = (unsigned long) rand();

	b = baseid;
	t = tmp;
	while (now) {
		*t++ = chars32[now & 0x1F];
		now = now >> 5;
	}

	while (t > tmp) {
		t--;
		*b++ = *t;
	}
	*b++ = '.';

	t = tmp;
	while (randy) {
		*t++ = chars32[randy & 0x1F];
		randy = randy >> 5;
	}
	while (t > tmp) {
		t--;
		*b++ = *t;
	}

	*b++ = '@';
	return (char *) baseid;
}


/*--------------------------------------------------------------------
 *
 *--------------------------------------------------------------------*/
gchar *
get_host_name (void)
{
	gchar *ptr;
	gchar hostname[MAXHOSTNAMELEN + 1];

	hostname[0] = '\0';

	if ((gethostname(hostname, sizeof(hostname))) == -1) {
		if ((ptr = getenv("HOST")) != NULL)
			strncpy(hostname, ptr, MAXHOSTNAMELEN);
		else {
			if ((ptr = getenv("HOSTNAME")) != NULL)
				strncpy(hostname, ptr, MAXHOSTNAMELEN);
			else
				hostname[0] = '\0';
		}
	}
	hostname[MAXHOSTNAMELEN] = '\0';
	ptr = strtok (hostname, ".");
	return g_strdup ( ptr );
}


gchar *
get_fqdn (gchar *host)
{
	char name[MAXHOSTNAMELEN + 2];
	char line[1025];
	char *cp, *domain;
	static char fqdn[1024];
	struct hostent *hp;
	struct in_addr in;
	FILE *inf;

	*fqdn = '\0';
	domain = NULL;

	name[MAXHOSTNAMELEN] = '\0';
	if (host) {
		if (strchr(host, '.'))
			return host;
		strncpy(name, host, MAXHOSTNAMELEN);
	}
	else {
		if (gethostname(name, MAXHOSTNAMELEN))
			return NULL;
	}

	if ('0' <= *name && *name <= '9') {
		in.s_addr = inet_addr(name);
		if ((hp = gethostbyaddr((char *) &in.s_addr, 4, AF_INET)))
			in.s_addr = (*hp->h_addr);
		return (hp && strchr(hp->h_name, '.') ? hp->h_name : (gchar *) inet_ntoa(in));
	}
	if ((hp = gethostbyname(name)) && !strchr(hp->h_name, '.'))
		if ((hp = gethostbyaddr(hp->h_addr, hp->h_length, hp->h_addrtype)))
			in.s_addr = (*hp->h_addr);

	sprintf(fqdn, "%s", hp ? strchr(hp->h_name, '.')
		? hp->h_name : (gchar *) inet_ntoa(in)
		: "");
	if (!*fqdn || (fqdn[strlen(fqdn) - 1] <= '9')) {
		*fqdn = '\0';
		inf = fopen("/etc/resolv.conf", "r");
		if (inf) {
			while (fgets(line, 1024, inf)) {
				line[1024] = '\0';
				(void) Pan_strtrim(line);
				if (strncmp(line, "domain ", 7) == 0) {
					domain = line + 7;
					break;
				}
				if (strncmp(line, "search ", 7) == 0) {
					domain = line + 7;
					cp = strchr(domain, ' ');
					if (cp)
						*cp = '\0';
					break;
				}
			}
			if (domain)
				sprintf(fqdn, "%s.%s", name, domain);
			fclose(inf);
		}
	}
	return (fqdn);
}


void
send_post (GtkWidget *widget, PostWindow *pw)
{
	gchar *subject, *from, *newsgroups, *message, *path, *organization;
	gchar *tmp;
	gchar *buf;
	static char *host = NULL;
	static char *uname = NULL;
	static char *fqdn = NULL;
	gchar *body;

	if (pw == NULL)
		g_error ("WTF!! pw == NULL\n");
	if (host == NULL)
		host = get_host_name ();
	if (fqdn == NULL)
		fqdn = get_fqdn (host);
	if (uname == NULL)
		uname = g_get_user_name ();

	path = g_strconcat (host, "!", uname, NULL);
	g_free ( host );

	from = gtk_entry_get_text (GTK_ENTRY(pw->from));
	newsgroups = gtk_entry_get_text (GTK_ENTRY(pw->newsgroups));
	subject = gtk_entry_get_text (GTK_ENTRY(pw->subject));
	organization = gtk_entry_get_text (GTK_ENTRY(pw->organization));

/* FIXME: doesn't do anything
	if (!check_from_syntax(from))
		g_error ("Bad From. Do something!");
	if (!check_newsgroups_syntax(newsgroups))
		g_error ("Bad Newsgroups. Do something!");
*/
	message = g_strconcat ("From: ", from, "\n",
			       "Path: ", path, "\n",
			       "Newsgroups: ", newsgroups, "\n",
			       "Subject: ", subject, "\n", NULL);
	g_free (path);
//	g_free (from);
//	g_free (newsgroups);
//	g_free (subject);
	tmp = message;
	
	if (Pan.active_server->gen_msgid) {
		message = g_strconcat (tmp,"Message-ID: <", generate_message_id(), fqdn, ">\n", NULL);
		g_free (tmp);
		tmp = message;
	}
	if (pw->article) {
		gchar* pch = generate_references ( pw );
		message = g_strconcat (message, "References: ", pch, "\n", NULL );
		g_free ( pch );
		g_free ( tmp );
		tmp = message;
	}
	body = g_strconcat ("\n", gtk_editable_get_chars(GTK_EDITABLE(pw->text), 0, -1), NULL);
	buf = generate_date ();
	message = g_strconcat (tmp,
			       "Date: ", buf, "\n",
			       "Organization: ", organization, "\n",
			       "X-Newsreader: ", "Pan ", VERSION, "\n",
			       body, NULL);
	g_free (buf);
	g_free (tmp);
	g_free (body);

	queue_add_post (message);
	g_free ( message );
	gtk_widget_destroy (pw->window);
}


GtkWidget *
create_info_pane (PostWindow *pw)
{
	GtkWidget *table;
	GtkWidget *label;
	gchar *p, *q, *r;

	table = gtk_table_new (4, 2, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);

	label = gtk_label_new (_("From:"));
	gtk_misc_set_alignment (GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, 4, 0);

	pw->from = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE(table), pw->from, 1, 2, 0, 1,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	if ((p = gnome_config_get_string ("/Pan/User/Email"))) {
		if ((q = gnome_config_get_string ("/Pan/User/Full_Name"))) {
			r = g_strdup_printf ("%s (%s)", p, q);
			g_free(q);
			g_free(p);
		} else {
			r = p;
		}
		gtk_entry_set_text (GTK_ENTRY(pw->from), r);
		g_free(r);
	}
	label = gtk_label_new (_("Organization:"));
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
			 GTK_FILL, GTK_FILL, 4, 0);

	pw->organization = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), pw->organization, 1, 2, 1, 2,
			 GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	if ((p = gnome_config_get_string ("/Pan/User/Organization"))) {
		gtk_entry_set_text(GTK_ENTRY(pw->organization), p);
		g_free(p);
	}
	label = gtk_label_new (_("Newsgroups:"));
	gtk_misc_set_alignment (GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 2, 3,
			  GTK_FILL, GTK_FILL, 4, 0);

	pw->newsgroups = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE(table), pw->newsgroups, 1, 2, 2, 3,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	if (pw->group && pw->group->name)
		gtk_entry_set_text (GTK_ENTRY(pw->newsgroups), pw->group->name);

	else if (Pan.active_group)
		gtk_entry_set_text (GTK_ENTRY(pw->newsgroups), Pan.active_group->name);

	label = gtk_label_new (_("Subject:"));
	gtk_misc_set_alignment (GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 3, 4,
			  GTK_FILL, GTK_FILL, 4, 0);

	pw->subject = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE(table), pw->subject, 1, 2, 3, 4,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);
	if (pw->article) {
		if ( ((pw->article->subject[0] == 'R')
		      || (pw->article->subject[0] == 'r'))
		     && ((pw->article->subject[1] == 'E')
			 || (pw->article->subject[1] == 'e'))
		     && (pw->article->subject[2] == ':')
		     && (pw->article->subject[3] == ' '))
		{
			gtk_entry_set_text (GTK_ENTRY(pw->subject), pw->article->subject);
		}
		else {
			gchar* pch = g_strconcat ( "Re: ", pw->article->subject, NULL );
			gtk_entry_set_text( GTK_ENTRY(pw->subject), pch );
			g_free ( pch );
		}
	}
	return table;
}


GtkWidget *
create_text_area (PostWindow *pw)
{
	gchar* sig_file;
	GtkWidget *scrolled_window;

	pw->text = gtk_text_new (NULL, NULL);
	gtk_text_set_editable (GTK_TEXT(pw->text), TRUE);
	gtk_text_set_word_wrap (GTK_TEXT(pw->text), TRUE);

	sig_file = gnome_config_get_string ( "/Pan/User/Signature_File" );
	if ( sig_file && strlen(sig_file) )
	{
		GArray* sig = read_file ( sig_file );
		if ( sig != NULL )
		{
			gtk_text_insert (GTK_TEXT(pw->text), NULL, &pw->text->style->black, NULL, "\n--\n\n", 4);
			gtk_text_insert (GTK_TEXT(pw->text), NULL, &pw->text->style->black, NULL, sig->data, sig->len);
			gtk_text_set_point (GTK_TEXT(pw->text), 0);
			g_array_free ( sig, TRUE );
		}
		g_free ( sig_file );
	}


	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER (scrolled_window), pw->text);

	return scrolled_window;
}


/*---[ post_window_spawn ]--------------------------------------------
 * spawn a 'post' window
 *--------------------------------------------------------------------*/
void
post_window_spawn (GtkWidget *widget, gint reply)
{
	PostWindow *pw;
	GtkWidget *vbox;
	gchar *body, *tmp, *tmp2, c;

	pw = g_malloc0 (sizeof(PostWindow));

	if (reply) {
		// Re: article subject
		pw->article = article_get_current();
		pw->window = gnome_app_new ("Post", _("Followup Message"));
	}
	else {
		// New Post
		pw->article = NULL;
		pw->window = gnome_app_new ("Post", _("New Message"));
	}

	gtk_signal_connect (GTK_OBJECT(pw->window), "destroy",
			    GTK_SIGNAL_FUNC (destroy_post_window),
			    pw);
	gtk_signal_connect (GTK_OBJECT(pw->window), "delete_event",
			    GTK_SIGNAL_FUNC (gtk_false), NULL);

	vbox = gtk_vbox_new (FALSE, 0);
	gnome_app_set_contents (GNOME_APP(pw->window), vbox);

	gtk_window_set_default_size (GTK_WINDOW (pw->window), 415, 300);
	gtk_window_set_position (GTK_WINDOW (pw->window), GTK_WIN_POS_CENTER);
	
	gnome_app_create_menus_with_data (GNOME_APP (pw->window), post_main_menu, pw);
	gnome_app_create_toolbar_with_data (GNOME_APP (pw->window), post_toolbar, pw);

	gtk_box_pack_start (GTK_BOX(vbox), create_info_pane (pw), FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), gtk_hseparator_new(), FALSE, FALSE, 2);
	gtk_box_pack_start (GTK_BOX(vbox), create_text_area (pw), TRUE, TRUE, 0);

	if (reply) {
		gtk_widget_realize (pw->text);
		gtk_text_freeze (GTK_TEXT(pw->text));
		tmp = g_strdup_printf (_("Nearly %d seconds after the Epoch, %s wrote:\n"),
				       (int) pw->article->date, pw->article->author);
		gtk_text_insert (GTK_TEXT(pw->text), NULL, &pw->text->style->black,
				 NULL, tmp, -1);
		g_free (tmp);

//		if (pw->article->body)
//			body = g_strdup (pw->article->body);
//		else
			body = gtk_editable_get_chars (GTK_EDITABLE (text), 0, -1);
		tmp = body;
		tmp2 = strchr(tmp, '\n');
		while (tmp2) {
			c = *(tmp2 + 1);
			*(tmp2 + 1) = '\0';
			gtk_text_insert (GTK_TEXT(pw->text), NULL,
					 &pw->text->style->black, NULL, "> ", 2);
			gtk_text_insert (GTK_TEXT(pw->text), NULL,
					 &pw->text->style->black, NULL, tmp, -1);
			*(tmp2 + 1) = c;
			tmp = tmp2 + 1;
			tmp2 = strchr (tmp, '\n');
		}
		g_free (body);
		gtk_text_thaw (GTK_TEXT(pw->text));
	}

	gtk_widget_show_all (pw->window);
}


void
followup_window (void)
{
	if (article_get_current() == NULL)
		return;
	else
		post_window_spawn (NULL, 1);
}
