/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "pan.h"


server_data*
server_fetch (char *name)
{
	server_data *sdata = NULL;
	gboolean has_section;

	gchar* buf = g_strdup_printf ( "/Pan/%s/", name );
	gnome_config_push_prefix ( buf );
	has_section = gnome_config_has_section ( buf );
	g_free ( buf );

	if ( has_section ) {
		sdata = (server_data *) g_malloc0(sizeof(server_data));
		if ((sdata->address = gnome_config_get_string ("Address")) == NULL) {
			g_free (sdata);
			sdata = NULL;
		}
		else {
			sdata->port = gnome_config_get_string_with_default ("Port=119", NULL);
			sdata->name = g_strdup(name);
			sdata->gen_msgid = gnome_config_get_int ("Generate_Msgid=0");
			sdata->need_auth = gnome_config_get_int ("Auth_Required=0");
			sdata->username = gnome_config_get_string ("Username");
			sdata->password = gnome_config_get_string ("Password");
		}
	}
	gnome_config_pop_prefix ();

	return sdata;
}


void
server_free (server_data *sdata)
{
	if (sdata) {
		g_free (sdata->name);
		g_free (sdata->address);
		g_free (sdata->port);
		g_free (sdata->username);
		g_free (sdata->password);
		g_free (sdata);
	}
}


void
server_init (void)
{

	/* This gets called before gtk_main and
	 * at the end of guru_finish */

	server_set_active ();
	server_menu_update ();
}


void
server_menu_update (void)
{
	int i;
	char *p;
	static char **sd = NULL;
	static int server_items = 0;
	GnomeUIInfo entry[2];

	gnome_app_remove_menus (GNOME_APP(Pan.window), "Servers/", server_items);
	server_items = 0;
	g_strfreev (sd);

	if ((p = gnome_config_get_string ("/Pan/Servers/Names")) == NULL)
		return;
    
	sd = g_strsplit (p, ",", 0);
	g_free (p);
	
	g_return_if_fail (sd);

	p = gnome_config_get_string ("/Pan/Servers/Active");
	for (i = 0; sd[i]; i++) {
		entry[0].type = GNOME_APP_UI_ITEM;
		entry[0].label = sd[i];
		entry[0].hint = NULL;
		entry[0].moreinfo = server_selected_cb;
		entry[0].user_data = sd[i];
		entry[0].unused_data = NULL;
		if (p && !strcmp(p, sd[i])) {
			entry[0].pixmap_type = GNOME_APP_PIXMAP_STOCK;
			entry[0].pixmap_info = GNOME_STOCK_PIXMAP_CLOSE;
		}
		else {
			entry[0].pixmap_type = GNOME_APP_PIXMAP_NONE;
			entry[0].pixmap_info = NULL;
		}
		entry[0].accelerator_key = 0;
		entry[1].type = GNOME_APP_UI_ENDOFINFO;
		gnome_app_insert_menus (GNOME_APP(Pan.window), "Servers/", entry);
		server_items++;
	}
	g_free (p);	
}


void
server_selected_cb (GtkWidget *menuitem, char *name)
{
	g_return_if_fail(name);

	if (is_online()) {
		gnome_ok_dialog (_("The active server cannot be changed while online."));
		return;
	}
	
	if (Pan.active_server && !strcmp(name, Pan.active_server->name))
		return;
	else {
		gnome_config_set_string ("/Pan/Servers/Active", name);
		gnome_config_sync ();
		server_set_active ();
		server_menu_update ();
	}
}


static void* db = 0;

void* server_get_attrib_db ( ) { return db; }

//============================================================================

void
server_set_active (void)
{
	char path[PATH_MAX];
	char *name;
	server_data *p, *q;

	if ( db )
	   pan_db_close ( db );
	
	name = gnome_config_get_string ("/Pan/Servers/Active");

	if (name) {
		g_snprintf (path, PATH_MAX, "/%s/%s/", data_dir, name);
		directory_check (path);

		q = Pan.active_server;
		p = server_fetch (name);
		Pan.active_server = p;
		if ( p != NULL )
		{
			gchar* filename = g_strdup_printf ( "/%s/%s.db", data_dir, p->name );
			db = pan_db_open ( filename );
			g_free (filename);
		}
		
		if ((!q) || (strcmp (p->name, q->name)))
		{
			/* Only do this stuff if it's a different server */
			group_set_active_group ( NULL );
			//article_ctree_clean ();
			//grouplist_free (Pan.grouplist);
			Pan.grouplist = NULL;
			grouplist_update ();
		}
		
		g_free (name);
		server_free (q);

	}
}
