/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "pan.h"

GtkWidget *text;
GtkWidget *text_rightclick_popup;

article_data *article;
//static char *current_article;

static GnomeUIInfo text_menu_popup[] = {
	{ GNOME_APP_UI_ITEM, N_("Reply to Sender"), N_("Create a reply to the sender."), followup_window },
	{ GNOME_APP_UI_ITEM, N_("Reply to _Group"), N_("Post a reply to the message on the news server."), followup_window },
	{ GNOME_APP_UI_ITEM, N_("Forward"), N_("Forward this message to someone."), not_implemented },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Show Full Header"), N_("Show the full header for the current message."), not_implemented },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Select All"), N_("Select all messages"), text_select_all,
	  NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'A', GDK_CONTROL_MASK, NULL},
	GNOMEUIINFO_END
};


void
text_select_all (void)
{
	gtk_editable_select_region (GTK_EDITABLE (text), 0, -1);
}


/*--------------------------------------------------------------------
 * called when the 'realize' signal is sent to the text widget,
 * not sure what it really does though... add comments ???
 *--------------------------------------------------------------------*/
void
realize_text_cb (GtkWidget *text, gpointer data)
{
	text_clear ();
	gtk_text_insert (GTK_TEXT(text), NULL, &text->style->black, NULL, "", -1);
}


/*---[ text_clear ]---------------------------------------------------
 * wipe the text widget clean
 *--------------------------------------------------------------------*/
void
text_clear (void)
{
	gint i;

	pan_lock();
	i = gtk_text_get_length (GTK_TEXT(text));
	if (i > 0) {
		gtk_text_set_point (GTK_TEXT(text), i);
		gtk_text_backward_delete (GTK_TEXT(text), i);
	}
	gtk_widget_queue_draw (GTK_WIDGET (text));
	pan_unlock ();
}


/*--------------------------------------------------------------------
 * callback for when a mouse button is pressed inside the text widget
 *--------------------------------------------------------------------*/
void
text_button_press (GtkWidget *widget, GdkEventButton *bevent)
{
	gtk_widget_grab_focus (GTK_WIDGET (text));

	if (bevent->button != 1)
		gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event");
	
	if (bevent->button == 3) {
		const int sel = (article) != NULL;

		GList* l = GTK_MENU_SHELL(text_rightclick_popup)->children; /* l is "reply to sender" */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );
		l = l->next; /* l is "reply to group" */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );
		l = l->next; /* l is "forward" */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );
		l = l->next; /* l is separator */
		l = l->next; /* l is "select all" */
		gtk_widget_set_sensitive ( GTK_WIDGET(l->data), sel );

		gtk_menu_popup (GTK_MENU(text_rightclick_popup), NULL, NULL, NULL,
				NULL, bevent->button, bevent->time);
	}
}


/*---[ text_create ]--------------------------------------------------
 * create a scrolled text area that displays a message's body
 *--------------------------------------------------------------------*/
GtkWidget *
text_create (void)
{
	GtkWidget *scrolled_window;

	text = gtk_text_new (NULL, NULL);

	widget_set_font ( GTK_WIDGET(text), message_body_font );

	gtk_text_set_editable (GTK_TEXT(text), FALSE);
	gtk_text_set_word_wrap (GTK_TEXT(text), TRUE);

	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	/* Add a handler to put a message in the text widget when it is realized */
	gtk_signal_connect (GTK_OBJECT(text), "realize",
			    GTK_SIGNAL_FUNC (realize_text_cb), NULL);

	gtk_signal_connect (GTK_OBJECT(text), "button_press_event",
			    GTK_SIGNAL_FUNC (text_button_press), NULL);

	gtk_container_add (GTK_CONTAINER (scrolled_window), text);

	text_rightclick_popup = gnome_popup_menu_new (text_menu_popup);

	return scrolled_window;

}


article_data *
displayed_article (void)
{
	return article;
}


/*--------------------------------------------------------------------
 *
 *--------------------------------------------------------------------*/

void
text_update (const gchar *message_id)
{
	char* body;
	article_data* adata = NULL;
	article_db db;

	/* update active message */
	if ( Pan.active_message_id )
		g_free ( Pan.active_message_id );
	Pan.active_message_id = g_strdup ( message_id );

	/* FIXME: Rethink an option to show full headers
	 * headers = gnome_config_get_int ("/Pan/Message/Headers=1"); */
	body = acache_load_body ( message_id );

	/* get the header information */
	db = ahdb_ref ( Pan.active_server, Pan.active_group );
	adata = ahdb_get ( db, message_id );
	ahdb_unref ( db );

	if (adata && body) {
		text_clear ();
		pan_lock ();
		
		//current_article = message_id;
		
		g_free (article);
		article = article_copy (adata);

		gtk_text_freeze (GTK_TEXT(text));

		gtk_label_set_text (GTK_LABEL (from_label), adata->author);
		gtk_label_set_text (GTK_LABEL (subject_label), adata->subject);

		gtk_text_insert (GTK_TEXT(text), NULL, &text->style->black, NULL, body, -1);

		gtk_text_thaw (GTK_TEXT(text));
		gtk_widget_queue_draw (GTK_WIDGET (text));

		pan_unlock ();

		g_free (adata);
		g_free (body);
	}
}


void
text_set_from_message_id (const gchar *message_id)
{
	if (acache_hit_body (message_id) == 0)
		text_update (message_id);
	else
		queue_add_article_download (Pan.active_server, Pan.active_group, message_id, QUEUE_FOREGROUND);
}

